@@ -38,6 +38,7 @@
 #include "map.h"
 #include "packets.h"
 #include "player.h"
+#include "sex.h"
 
 /* client */
 #include "client_main.h"
@@ -96,11 +97,11 @@ static int selected_style;
 
 static int is_showing_pillage_dialog = FALSE;
 
-/**************************************************************************
+/**********************************************************************//**
   Popup a generic dialog to display some generic information.
 **************************************************************************/
 void popup_notify_dialog(const char *caption, const char *headline,
-			 const char *lines)
+                         const char *lines)
 {
   static struct gui_dialog *shell;
   GtkWidget *vbox, *label, *headline_label, *sw;
@@ -118,7 +119,7 @@ void popup_notify_dialog(const char *caption, const char *headline,
   gtk_grid_set_row_spacing(GTK_GRID(vbox), 2);
   gtk_container_add(GTK_CONTAINER(shell->vbox), vbox);
 
-  headline_label = gtk_label_new(headline);   
+  headline_label = gtk_label_new(headline);
   gtk_container_add(GTK_CONTAINER(vbox), headline_label);
   gtk_widget_set_name(headline_label, "notify_label");
 
@@ -151,10 +152,10 @@ void popup_notify_dialog(const char *caption, const char *headline,
   shell = NULL;
 }
 
-/****************************************************************
+/**********************************************************************//**
   User has responded to notify dialog with possibility to
   center (goto) on event location.
-*****************************************************************/
+**************************************************************************/
 static void notify_goto_response(GtkWidget *w, gint response)
 {
   struct city *pcity = NULL;
@@ -179,15 +180,15 @@ static void notify_goto_response(GtkWidget *w, gint response)
   gtk_widget_destroy(w);
 }
 
-/****************************************************************
+/**********************************************************************//**
   User clicked close for connect message dialog
-*****************************************************************/
+**************************************************************************/
 static void notify_connect_msg_response(GtkWidget *w, gint response)
 {
   gtk_widget_destroy(w);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Popup a dialog to display information about an event that has a
   specific location.  The user should be given the option to goto that
   location.
@@ -197,7 +198,7 @@ void popup_notify_goto_dialog(const char *headline, const char *lines,
                               struct tile *ptile)
 {
   GtkWidget *shell, *label;
-  
+
   if (ptile == NULL) {
     shell = gtk_dialog_new_with_buttons(headline, NULL, 0,
                                         _("_Close"), GTK_RESPONSE_CLOSE,
@@ -232,13 +233,13 @@ void popup_notify_goto_dialog(const char *headline, const char *lines,
   gtk_widget_show(shell);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Popup a dialog to display connection message from server.
 **************************************************************************/
 void popup_connect_msg(const char *headline, const char *message)
 {
   GtkWidget *shell, *label;
-  
+
   shell = gtk_dialog_new();
   gtk_window_set_title(GTK_WINDOW(shell), headline);
   setup_dialog(shell, toplevel);
@@ -258,9 +259,9 @@ void popup_connect_msg(const char *headline, const char *message)
   gtk_widget_show(shell);
 }
 
-/****************************************************************
+/**********************************************************************//**
   User has responded to revolution dialog
-*****************************************************************/
+**************************************************************************/
 static void revolution_response(GtkWidget *w, gint response, gpointer data)
 {
   struct government *government = data;
@@ -277,9 +278,9 @@ static void revolution_response(GtkWidget *w, gint response, gpointer data)
   }
 }
 
-/****************************************************************
+/**********************************************************************//**
   Popup revolution dialog for user
-*****************************************************************/
+**************************************************************************/
 void popup_revolution_dialog(struct government *government)
 {
   static GtkWidget *shell = NULL;
@@ -331,9 +332,9 @@ static void pillage_callback(GtkWidget *dlg, gint arg)
   gtk_widget_destroy(dlg);
 }
 
-/****************************************************************
+/**********************************************************************//**
   Opens pillage dialog listing possible pillage targets.
-*****************************************************************/
+**************************************************************************/
 void popup_pillage_dialog(struct unit *punit, bv_extras extras)
 {
   if (!is_showing_pillage_dialog) {
@@ -369,32 +370,32 @@ void popup_pillage_dialog(struct unit *punit, bv_extras extras)
   }
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Popup unit selection dialog. It is a wrapper for the main function; see
   unitselect.c:unit_select_dialog_popup_main().
-*****************************************************************************/
+**************************************************************************/
 void unit_select_dialog_popup(struct tile *ptile)
 {
   unit_select_dialog_popup_main(ptile, TRUE);
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Update unit selection dialog. It is a wrapper for the main function; see
   unitselect.c:unit_select_dialog_popup_main().
-*****************************************************************************/
+**************************************************************************/
 void unit_select_dialog_update_real(void *unused)
 {
   unit_select_dialog_popup_main(NULL, FALSE);
 }
 
-/*****************************************************************************
+/**************************************************************************
   NATION SELECTION DIALOG
-*****************************************************************************/
-/*****************************************************************************
+**************************************************************************/
+/**********************************************************************//**
   Return the GtkTreePath for a given nation on the specified list, or NULL
   if it's not there at all.
   Caller must free with gtk_tree_path_free().
-*****************************************************************************/
+**************************************************************************/
 static GtkTreePath *path_to_nation_on_list(Nation_type_id nation,
                                            GtkTreeView *list)
 {
@@ -417,18 +418,19 @@ static GtkTreePath *path_to_nation_on_list(Nation_type_id nation,
   }
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Make sure the given nation is selected in the list on a given groups
   notebook tab, if it's present on that tab.
   Intended for synchronising the tabs to the current selection, so does not
   disturb the controls on the right-hand side.
-*****************************************************************************/
+**************************************************************************/
 static void select_nation_on_tab(GtkWidget *tab_list, int nation)
 {
   /* tab_list is a GtkTreeView (not its enclosing GtkScrolledWindow). */
   GtkTreeView *list = GTK_TREE_VIEW(tab_list);
   GtkTreeSelection *select = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
   GtkTreePath *path = path_to_nation_on_list(nation, list);
+
   /* Suppress normal effects of selection change to avoid loops. */
   g_signal_handlers_block_by_func(select, races_nation_callback, NULL);
   if (path) {
@@ -463,13 +465,14 @@ static void select_nation_on_tab(GtkWidget *tab_list, int nation)
   g_signal_handlers_unblock_by_func(select, races_nation_callback, NULL);
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Select the given nation in the nation lists in the left-hand-side notebook.
-*****************************************************************************/
+**************************************************************************/
 static void sync_tabs_to_nation(int nation)
 {
   /* Ensure that all tabs are in sync with the new selection */
   int i;
+
   for (i = 0; i <= nation_group_count(); i++) {
     if (races_nation_list[i]) {
       select_nation_on_tab(races_nation_list[i], nation);
@@ -477,10 +480,10 @@ static void sync_tabs_to_nation(int nation)
   }
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Populates leader list.
   If no nation selected, blanks it.
-*****************************************************************************/
+**************************************************************************/
 static void populate_leader_list(void)
 {
   int i;
@@ -501,12 +504,12 @@ static void populate_leader_list(void)
   }
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Update dialog state by selecting a nation and choosing values for its
   parameters, and update the right-hand side of the dialog accordingly.
   If 'leadername' is NULL, pick a random leader name and sex from the
   nation's list (ignoring the 'is_male' parameter).
-*****************************************************************************/
+**************************************************************************/
 static void select_nation(int nation,
                           const char *leadername, bool is_male,
                           int style_id)
@@ -528,6 +531,7 @@ static void select_nation(int nation,
     } else {
       int idx = fc_rand(nation_leader_list_size(
                         nation_leaders(nation_by_number(selected_nation))));
+
       gtk_combo_box_set_active(GTK_COMBO_BOX(races_leader), idx);
       /* This also updates the leader sex, eventually. */
     }
@@ -558,6 +562,7 @@ static void select_nation(int nation,
     /* Update nation description. */
     {
       char buf[4096];
+
       helptext_nation(buf, sizeof(buf),
                       nation_by_number(selected_nation), NULL);
       gtk_text_buffer_set_text(races_text, buf, -1);
@@ -578,6 +583,7 @@ static void select_nation(int nation,
     {
       GtkTreeSelection* select
         = gtk_tree_view_get_selection(GTK_TREE_VIEW(races_style_list));
+
       gtk_tree_selection_unselect_all(select);
     }
     /* Nation description */
@@ -594,12 +600,12 @@ static void select_nation(int nation,
   sync_tabs_to_nation(selected_nation);
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Creates a list of currently-pickable nations in the given group
   Inserts appropriate gtk_tree_view into races_nation_list[index] (or NULL if
   the group has no nations)
   If group == NULL, create a list of all nations
-*****************************************************************************/
+**************************************************************************/
 static GtkWidget* create_list_of_nations_in_group(struct nation_group* group,
 						  int index)
 {
@@ -653,7 +659,7 @@ static GtkWidget* create_list_of_nations_in_group(struct nation_group* group,
       gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
           GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
       gtk_container_add(GTK_CONTAINER(sw), list);
-     
+
       render = gtk_cell_renderer_pixbuf_new();
       column = gtk_tree_view_column_new_with_attributes("Flag", render,
           "pixbuf", 2, NULL);
@@ -684,16 +690,16 @@ static GtkWidget* create_list_of_nations_in_group(struct nation_group* group,
   return sw;
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Creates lists of nations for left side of nation selection dialog
-*****************************************************************************/
+**************************************************************************/
 static void create_nation_selection_lists(void)
 {
   GtkWidget *nation_list;
   GtkWidget *group_name_label;
-  
+
   int i;
-  
+
   for (i = 0; i < nation_group_count(); i++) {
     struct nation_group* group = (nation_group_by_number(i));
     if (is_nation_group_hidden(group)) {
@@ -707,7 +713,7 @@ static void create_nation_selection_lists(void)
                                group_name_label);
     }
   }
-  
+
   nation_list = create_list_of_nations_in_group(NULL, nation_group_count());
   /* Even this list can be empty if there are no pickable nations (due to
    * a combination of start position and nationset restrictions). */
@@ -718,10 +724,10 @@ static void create_nation_selection_lists(void)
   }
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   The server has changed the set of selectable nations.
   Update any current nations dialog accordingly.
-*****************************************************************************/
+**************************************************************************/
 void races_update_pickable(bool nationset_change)
 {
   int tab, groupidx;
@@ -789,27 +795,29 @@ void races_update_pickable(bool nationset_change)
   sync_tabs_to_nation(selected_nation);
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Sync nationset control with the current state of the server.
-*****************************************************************************/
+**************************************************************************/
 void nationset_sync_to_server(const char *nationset)
 {
   if (nationsets_chooser) {
     struct nation_set *set = nation_set_by_setting_value(nationset);
+
     gtk_combo_box_set_active(GTK_COMBO_BOX(nationsets_chooser),
                              nation_set_index(set));
   }
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Called when the nationset control's value has changed.
-*****************************************************************************/
+**************************************************************************/
 static void nationset_callback(GtkComboBox *b, gpointer data)
 {
   GtkTreeIter iter;
   if (gtk_combo_box_get_active_iter(b, &iter)) {
     struct option *poption = optset_option_by_name(server_optset, "nationset");
     gchar *rule_name;
+
     gtk_tree_model_get(gtk_combo_box_get_model(b), &iter,
                        0, &rule_name, -1);
     /* Suppress propagation of an option value equivalent to the current
@@ -825,9 +833,9 @@ static void nationset_callback(GtkComboBox *b, gpointer data)
   }
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Create nations dialog
-*****************************************************************************/
+**************************************************************************/
 static void create_races_dialog(struct player *pplayer)
 {
   GtkWidget *shell;
@@ -836,13 +844,11 @@ static void create_races_dialog(struct player *pplayer)
   GtkWidget *frame, *label, *combo;
   GtkWidget *text;
   GtkWidget *notebook;
-  
   GtkWidget *sw;
-  GtkWidget *list;  
+  GtkWidget *list;
   GtkListStore *store;
   GtkCellRenderer *render;
   GtkTreeViewColumn *column;
-  
   int i;
   char *title;
 
@@ -904,6 +910,7 @@ static void create_races_dialog(struct player *pplayer)
         gchar *escaped;
         struct astring s = ASTRING_INIT;
         int num_nations = 0;
+
         nations_iterate(pnation) {
           if (is_nation_playable(pnation) && nation_is_in_set(pnation, pset)) {
             num_nations++;
@@ -922,11 +929,15 @@ static void create_races_dialog(struct player *pplayer)
            * to specify the indentation we want. So we do it ourselves. */
           char *desc = fc_strdup(_(nation_set_description(pset)));
           char *p = desc;
+
           fc_break_lines(desc, 70);
           astr_add(&s, "\n");
           while (*p) {
             int len = strcspn(p, "\n");
-            if (p[len] == '\n') len++;
+
+            if (p[len] == '\n') {
+              len++;
+            }
             escaped = g_markup_escape_text(p, len);
             astr_add(&s, "\t%s", escaped);
             g_free(escaped);
@@ -952,6 +963,7 @@ static void create_races_dialog(struct player *pplayer)
          * like a cell-view: disable editing, and focusing (which removes
          * the caret). */
         GtkWidget *entry = gtk_bin_get_child(GTK_BIN(nationsets_chooser));
+
         gtk_editable_set_editable(GTK_EDITABLE(entry), FALSE);
         gtk_widget_set_can_focus(entry, FALSE);
       }
@@ -994,13 +1006,14 @@ static void create_races_dialog(struct player *pplayer)
     }
 
     races_notebook = gtk_notebook_new();
-    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(races_notebook), GTK_POS_LEFT);  
+    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(races_notebook), GTK_POS_LEFT);
     gtk_grid_attach(GTK_GRID(nation_selection_list), races_notebook,
                     0, 2, 2, 1);
 
     /* Suppress notebook tabs if there will be only one ("All") */
     {
       bool show_groups = FALSE;
+
       nation_groups_iterate(pgroup) {
         if (!is_nation_group_hidden(pgroup)) {
           show_groups = TRUE;
@@ -1044,9 +1057,10 @@ static void create_races_dialog(struct player *pplayer)
   gtk_grid_set_row_spacing(GTK_GRID(table), 2);
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, label);
 
-  /* Leader. */ 
+  /* Leader. */
   {
     GtkListStore *model = gtk_list_store_new(1, G_TYPE_STRING);
+
     combo = gtk_combo_box_new_with_model_and_entry(GTK_TREE_MODEL(model));
     gtk_combo_box_set_entry_text_column(GTK_COMBO_BOX(combo), 0);
     g_object_unref(G_OBJECT(model));
@@ -1064,13 +1078,15 @@ static void create_races_dialog(struct player *pplayer)
   gtk_grid_attach(GTK_GRID(table), label, 0, 0, 1, 2);
   gtk_grid_attach(GTK_GRID(table), combo, 1, 0, 2, 1);
 
-  cmd = gtk_radio_button_new_with_mnemonic(NULL, _("_Female"));
+  cmd = gtk_radio_button_new_with_mnemonic(NULL,
+                                           sex_name_mnemonic(SEX_FEMALE, "_"));
   gtk_widget_set_margin_bottom(cmd, 6);
   races_sex[0] = cmd;
   gtk_grid_attach(GTK_GRID(table), cmd, 1, 1, 1, 1);
 
   cmd = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(cmd),
-      _("_Male"));
+                                                       sex_name_mnemonic(SEX_MALE,
+                                                                         "_"));
   gtk_widget_set_margin_bottom(cmd, 6);
   races_sex[1] = cmd;
   gtk_grid_attach(GTK_GRID(table), cmd, 2, 1, 1, 1);
@@ -1163,9 +1179,9 @@ static void create_races_dialog(struct player *pplayer)
       G_CALLBACK(races_leader_callback), NULL);
 
   g_signal_connect(races_sex[0], "toggled",
-      G_CALLBACK(races_sex_callback), GINT_TO_POINTER(0));
+      G_CALLBACK(races_sex_callback), GINT_TO_POINTER(SEX_FEMALE));
   g_signal_connect(races_sex[1], "toggled",
-      G_CALLBACK(races_sex_callback), GINT_TO_POINTER(1));
+      G_CALLBACK(races_sex_callback), GINT_TO_POINTER(SEX_MALE));
 
   /* Finish up. */
   gtk_dialog_set_default_response(GTK_DIALOG(shell), GTK_RESPONSE_CANCEL);
@@ -1193,9 +1209,9 @@ static void create_races_dialog(struct player *pplayer)
   }
 }
 
-/****************************************************************
-  popup the dialog 10% inside the main-window 
- *****************************************************************/
+/**********************************************************************//**
+  Popup the dialog 10% inside the main-window
+**************************************************************************/
 void popup_races_dialog(struct player *pplayer)
 {
   if (!pplayer) {
@@ -1208,9 +1224,9 @@ void popup_races_dialog(struct player *pplayer)
   }
 }
 
-/****************************************************************
-  Close nations dialog  
-*****************************************************************/
+/**********************************************************************//**
+  Close nations dialog
+**************************************************************************/
 void popdown_races_dialog(void)
 {
   if (races_shell) {
@@ -1222,7 +1238,7 @@ void popdown_races_dialog(void)
   blank_max_unit_size();
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Update which nations are allowed to be selected (due to e.g. another
   player choosing a nation).
 **************************************************************************/
@@ -1271,9 +1287,9 @@ void races_toggles_set_sensitive(void)
   }
 }
 
-/*****************************************************************************
+/**********************************************************************//**
   Called whenever a user selects a nation in nation list
-*****************************************************************************/
+**************************************************************************/
 static void races_nation_callback(GtkTreeSelection *select, gpointer data)
 {
   GtkTreeModel *model;
@@ -1300,7 +1316,7 @@ static void races_nation_callback(GtkTreeSelection *select, gpointer data)
   select_nation(-1, NULL, FALSE, 0);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Leader name has been chosen
 **************************************************************************/
 static void races_leader_callback(void)
@@ -1320,7 +1336,7 @@ static void races_leader_callback(void)
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Leader sex has been chosen
 **************************************************************************/
 static void races_sex_callback(GtkWidget *w, gpointer data)
@@ -1328,12 +1344,12 @@ static void races_sex_callback(GtkWidget *w, gpointer data)
   selected_sex = GPOINTER_TO_INT(data);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Determines which nations can be selected in the UI
 **************************************************************************/
 static gboolean races_selection_func(GtkTreeSelection *select,
-				     GtkTreeModel *model, GtkTreePath *path,
-				     gboolean selected, gpointer data)
+                                     GtkTreeModel *model, GtkTreePath *path,
+                                     gboolean selected, gpointer data)
 {
   GtkTreeIter it;
   gboolean chosen;
@@ -1343,7 +1359,7 @@ static gboolean races_selection_func(GtkTreeSelection *select,
   return (!chosen || selected);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   City style has been chosen
 **************************************************************************/
 static void races_style_callback(GtkTreeSelection *select, gpointer data)
@@ -1358,7 +1374,7 @@ static void races_style_callback(GtkTreeSelection *select, gpointer data)
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   User has selected some of the responses for whole nations dialog
 **************************************************************************/
 static void races_response(GtkWidget *w, gint response, gpointer data)
@@ -1403,19 +1419,19 @@ static void races_response(GtkWidget *w, gint response, gpointer data)
   popdown_races_dialog();
 }
 
-
-/**************************************************************************
+/**********************************************************************//**
   Adjust tax rates from main window
 **************************************************************************/
-gboolean taxrates_callback(GtkWidget * w, GdkEventButton * ev, gpointer data)
+gboolean taxrates_callback(GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
-  common_taxrates_callback((size_t) data);
+  common_taxrates_callback((size_t) data, FALSE);
+
   return TRUE;
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Pops up a dialog to confirm upgrading of the unit.
-****************************************************************************/
+**************************************************************************/
 void popup_upgrade_dialog(struct unit_list *punits)
 {
   GtkWidget *shell;
@@ -1451,9 +1467,9 @@ void popup_upgrade_dialog(struct unit_list *punits)
   }
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Pops up a dialog to confirm disband of the unit(s).
-****************************************************************************/
+**************************************************************************/
 void popup_disband_dialog(struct unit_list *punits)
 {
   GtkWidget *shell;
@@ -1491,10 +1507,10 @@ void popup_disband_dialog(struct unit_list *punits)
   }
 }
 
-/********************************************************************** 
+/**********************************************************************//**
   This function is called when the client disconnects or the game is
   over.  It should close all dialog windows for that game.
-***********************************************************************/
+**************************************************************************/
 void popdown_all_game_dialogs(void)
 {
   gui_dialog_destroy_all();
@@ -1502,9 +1518,9 @@ void popdown_all_game_dialogs(void)
   unit_select_dialog_popdown();
 }
 
-/****************************************************************
+/**********************************************************************//**
   Player has gained a new tech.
-*****************************************************************/
+**************************************************************************/
 void show_tech_gained_dialog(Tech_type_id tech)
 {
   const struct advance *padvance = valid_advance_by_number(tech);
@@ -1517,10 +1533,10 @@ void show_tech_gained_dialog(Tech_type_id tech)
   }
 }
 
-/****************************************************************
+/**********************************************************************//**
   Show tileset error dialog. It's blocking as client will
   shutdown as soon as this function returns.
-*****************************************************************/
+**************************************************************************/
 void show_tileset_error(bool fatal, const char *tset_name, const char *msg)
 {
   if (is_gui_up()) {
@@ -1550,22 +1566,88 @@ void show_tileset_error(bool fatal, const char *tset_name, const char *msg)
   }
 }
 
-/****************************************************************
+/**********************************************************************//**
   Give a warning when user is about to edit scenario with manually
   set properties.
-*****************************************************************/
+**************************************************************************/
 bool handmade_scenario_warning(void)
 {
   /* Just tell the client common code to handle this. */
   return FALSE;
 }
 
-/***************************************************************************
+/**********************************************************************//**
   Popup detailed information about battle or save information for
   some kind of statistics
-***************************************************************************/
+**************************************************************************/
 void popup_combat_info(int attacker_unit_id, int defender_unit_id,
                        int attacker_hp, int defender_hp,
                        bool make_att_veteran, bool make_def_veteran)
 {
 }
+
+/**********************************************************************//**
+  This is the response callback for the action confirmation dialog.
+**************************************************************************/
+static void act_conf_response(GtkWidget *dialog, gint response,
+                              gpointer data)
+{
+  gtk_widget_destroy(dialog);
+
+  if (response == GTK_RESPONSE_YES) {
+    action_confirmation(data, TRUE);
+  } else {
+    action_confirmation(data, FALSE);
+  }
+}
+
+/**********************************************************************//**
+  Common code wants confirmation for an action.
+**************************************************************************/
+void request_action_confirmation(const char *expl,
+                                 struct act_confirmation_data *data)
+{
+  GtkWidget *dialog;
+  char buf[1024];
+
+  if (expl != NULL) {
+    fc_snprintf(buf, sizeof(buf), _("Are you sure you want to do %s?\n%s"),
+                action_id_name_translation(data->act), expl);
+  } else {
+    fc_snprintf(buf, sizeof(buf), _("Are you sure you want to do %s?"),
+                action_id_name_translation(data->act));
+  }
+
+  dialog = gtk_message_dialog_new(NULL, 0,
+                                  GTK_MESSAGE_WARNING,
+                                  GTK_BUTTONS_YES_NO,
+                                  "%s", buf);
+  setup_dialog(dialog, toplevel);
+
+  g_signal_connect(dialog, "response",
+                   G_CALLBACK(act_conf_response), data);
+
+  gtk_window_present(GTK_WINDOW(dialog));
+}
+
+/**********************************************************************//**
+  Popup image window
+**************************************************************************/
+void popup_image(const char *tag)
+{
+  struct sprite *spr = load_popup_sprite(tag);
+
+  if (spr != NULL) {
+    GdkPixbuf *pix = sprite_get_pixbuf(spr);
+    GtkWidget *win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    GtkWidget *img = gtk_image_new_from_pixbuf(pix);
+
+    gtk_container_add(GTK_CONTAINER(win), img);
+    gtk_widget_show(img);
+    gtk_widget_show(win);
+
+    unload_popup_sprite(tag);
+  } else {
+    log_error(_("No image for tag \"%s\", requested by the server."), tag);
+  }
+}
