@@ -33,8 +33,9 @@
 
 /* common */
 #include "game.h"
-#include "government.h"		/* government_graphic() */
+#include "government.h"	        /* government_graphic() */
 #include "map.h"
+#include "nation.h"
 #include "player.h"
 
 /* client */
@@ -50,7 +51,7 @@
 #include "text.h"
 #include "zoom.h"
 
-/* client/gui-gtk-3.0 */
+/* client/gui-gtk-3.22 */
 #include "citydlg.h" /* For reset_city_dialogs() */
 #include "editgui.h"
 #include "graphics.h"
@@ -66,7 +67,7 @@ static GtkAdjustment *map_hadj, *map_vadj;
 static int cursor_timer_id = 0, cursor_type = -1, cursor_frame = 0;
 static int mapview_frozen_level = 0;
 
-/**************************************************************************
+/**********************************************************************//**
   If do_restore is FALSE it will invert the turn done button style. If
   called regularly from a timer this will give a blinking turn done
   button. If do_restore is TRUE this will reset the turn done button
@@ -81,21 +82,37 @@ void update_turn_done_button(bool do_restore)
   }
 
   if ((do_restore && flip) || !do_restore) {
-    GdkRGBA fore;
-    GdkRGBA back;
-    GtkStyleContext *context = gtk_widget_get_style_context(turn_done_button);
+    static GtkCssProvider *tdb_provider = NULL;
+    GtkStyleContext *scontext = gtk_widget_get_style_context(turn_done_button);
 
-    gtk_style_context_get_color(context, GTK_STATE_FLAG_NORMAL, &fore);
-    gtk_style_context_get_background_color(context, GTK_STATE_FLAG_NORMAL, &back);
+    if (tdb_provider == NULL) {
+      tdb_provider = gtk_css_provider_new();
 
-    gtk_widget_override_color(turn_done_button, GTK_STATE_FLAG_NORMAL, &back);
-    gtk_widget_override_background_color(turn_done_button, GTK_STATE_FLAG_NORMAL, &fore);
+      gtk_css_provider_load_from_data(tdb_provider,
+                                      ".lighted {\n"
+                                      "color: rgba(235, 127, 235, 255);\n"
+                                      "background-color: rgba(127, 127, 127, 255);\n"
+                                      "}\n",
+                                      -1, NULL);
+
+      /* Turn Done button is persistent, so we only need to do this
+       * once too. */
+      gtk_style_context_add_provider(scontext,
+                                     GTK_STYLE_PROVIDER(tdb_provider),
+                                     GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+    }
+
+    if (flip) {
+      gtk_style_context_add_class(scontext, "lighted");
+    } else {
+      gtk_style_context_remove_class(scontext, "lighted");
+    }
 
     flip = !flip;
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Timeout label requires refreshing
 **************************************************************************/
 void update_timeout_label(void)
@@ -115,7 +132,7 @@ void update_timeout_label(void)
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Refresh info label
 **************************************************************************/
 void update_info_label(void)
@@ -208,8 +225,8 @@ void update_info_label(void)
   gtk_widget_set_tooltip_text(government_ebox, get_government_tooltip());
 }
 
-/**************************************************************************
-  This function is used to animate the mouse cursor. 
+/**********************************************************************//**
+  This function is used to animate the mouse cursor.
 **************************************************************************/
 static gboolean anim_cursor_cb(gpointer data)
 {
@@ -225,7 +242,7 @@ static gboolean anim_cursor_cb(gpointer data)
   if (cursor_type == CURSOR_DEFAULT) {
     gdk_window_set_cursor(root_window, NULL);
     cursor_timer_id = 0;
-    return FALSE; 
+    return FALSE;
   }
 
   gdk_window_set_cursor(root_window,
@@ -234,7 +251,7 @@ static gboolean anim_cursor_cb(gpointer data)
   return TRUE;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   This function will change the current mouse cursor.
 **************************************************************************/
 void update_mouse_cursor(enum cursor_type new_cursor_type)
@@ -245,7 +262,7 @@ void update_mouse_cursor(enum cursor_type new_cursor_type)
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Update the information label which gives info on the current unit and the
   square under the current unit, for specified unit.  Note that in practice
   punit is always the focus unit.
@@ -269,7 +286,7 @@ void update_unit_info_label(struct unit_list *punits)
   update_unit_pix_label(punits);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Get sprite for treaty acceptance or rejection.
 **************************************************************************/
 GdkPixbuf *get_thumb_pixbuf(int onoff)
@@ -277,11 +294,11 @@ GdkPixbuf *get_thumb_pixbuf(int onoff)
   return sprite_get_pixbuf(get_treaty_thumb_sprite(tileset, BOOL_VAL(onoff)));
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Set information for the indicator icons typically shown in the main
   client window.  The parameters tell which sprite to use for the
   indicator.
-****************************************************************************/
+**************************************************************************/
 void set_indicator_icons(struct sprite *bulb, struct sprite *sol,
                          struct sprite *flake, struct sprite *gov)
 {
@@ -291,19 +308,19 @@ void set_indicator_icons(struct sprite *bulb, struct sprite *sol,
   gtk_image_set_from_surface(GTK_IMAGE(government_label), gov->surface);
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Return the maximum dimensions of the area (container widget) for the
   overview. Due to the fact that the scaling factor is at least 1, the real
   size could be larger. The calculation in calculate_overview_dimensions()
   limit it to the smallest possible size.
-****************************************************************************/
+**************************************************************************/
 void get_overview_area_dimensions(int *width, int *height)
 {
   *width = GUI_GTK_OVERVIEW_MIN_XSIZE;
   *height = GUI_GTK_OVERVIEW_MIN_YSIZE;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Size of overview changed
 **************************************************************************/
 void overview_size_changed(void)
@@ -314,9 +331,9 @@ void overview_size_changed(void)
   update_map_canvas_scrollbars_size();
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Return a canvas that is the overview window.
-****************************************************************************/
+**************************************************************************/
 struct canvas *get_overview_window(void)
 {
 #if 0
@@ -333,7 +350,7 @@ struct canvas *get_overview_window(void)
   return NULL;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Redraw overview canvas
 **************************************************************************/
 gboolean overview_canvas_draw(GtkWidget *w, cairo_t *cr, gpointer data)
@@ -350,17 +367,17 @@ gboolean overview_canvas_draw(GtkWidget *w, cairo_t *cr, gpointer data)
   return TRUE;
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Freeze the drawing of the map.
-****************************************************************************/
+**************************************************************************/
 void mapview_freeze(void)
 {
   mapview_frozen_level++;
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Thaw the drawing of the map.
-****************************************************************************/
+**************************************************************************/
 void mapview_thaw(void)
 {
   if (1 < mapview_frozen_level) {
@@ -372,15 +389,15 @@ void mapview_thaw(void)
   }
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Return whether the map should be drawn or not.
-****************************************************************************/
+**************************************************************************/
 bool mapview_is_frozen(void)
 {
   return (0 < mapview_frozen_level);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Update on canvas widget size change
 **************************************************************************/
 gboolean map_canvas_configure(GtkWidget *w, GdkEventConfigure *ev,
@@ -391,7 +408,17 @@ gboolean map_canvas_configure(GtkWidget *w, GdkEventConfigure *ev,
   return TRUE;
 }
 
-/**************************************************************************
+/**********************************************************************//**
+  Refresh map canvas size information
+**************************************************************************/
+void map_canvas_size_refresh(void)
+{
+  /* Needed only with full screen zoom mode.
+   * Not needed, nor implemented, in this client. */
+  fc_assert(FALSE);
+}
+
+/**********************************************************************//**
   Redraw map canvas.
 **************************************************************************/
 gboolean map_canvas_draw(GtkWidget *w, cairo_t *cr, gpointer data)
@@ -409,20 +436,7 @@ gboolean map_canvas_draw(GtkWidget *w, cairo_t *cr, gpointer data)
   return TRUE;
 }
 
-/**************************************************************************
-  Flush the given part of the canvas buffer (if there is one) to the
-  screen.
-**************************************************************************/
-void flush_mapcanvas(int canvas_x, int canvas_y,
-                     int pixel_width, int pixel_height)
-{
-  GdkRectangle rectangle = {canvas_x, canvas_y, pixel_width, pixel_height};
-  if (gtk_widget_get_realized(map_canvas) && !mapview_is_frozen()) {
-    gdk_window_invalidate_rect(gtk_widget_get_window(map_canvas), &rectangle, FALSE);
-  }
-}
-
-/**************************************************************************
+/**********************************************************************//**
   Mark the rectangular region as "dirty" so that we know to flush it
   later.
 **************************************************************************/
@@ -430,12 +444,13 @@ void dirty_rect(int canvas_x, int canvas_y,
                 int pixel_width, int pixel_height)
 {
   GdkRectangle rectangle = {canvas_x, canvas_y, pixel_width, pixel_height};
+
   if (gtk_widget_get_realized(map_canvas)) {
     gdk_window_invalidate_rect(gtk_widget_get_window(map_canvas), &rectangle, FALSE);
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Mark the entire screen area as "dirty" so that we can flush it later.
 **************************************************************************/
 void dirty_all(void)
@@ -445,7 +460,7 @@ void dirty_all(void)
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Flush all regions that have been previously marked as dirty.  See
   dirty_rect and dirty_all.  This function is generally called after we've
   processed a batch of drawing operations.
@@ -457,25 +472,25 @@ void flush_dirty(void)
   }
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Do any necessary synchronization to make sure the screen is up-to-date.
   The canvas should have already been flushed to screen via flush_dirty -
   all this function does is make sure the hardware has caught up.
-****************************************************************************/
+**************************************************************************/
 void gui_flush(void)
 {
   cairo_surface_flush(mapview.store->surface);
 }
 
-/**************************************************************************
- Update display of descriptions associated with cities on the main map.
+/**********************************************************************//**
+  Update display of descriptions associated with cities on the main map.
 **************************************************************************/
 void update_city_descriptions(void)
 {
   update_map_canvas_visible();
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Fill image with unit gfx
 **************************************************************************/
 void put_unit_image(struct unit *punit, GtkImage *p, int height)
@@ -497,11 +512,11 @@ void put_unit_image(struct unit *punit, GtkImage *p, int height)
   cairo_surface_destroy(store.surface);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   FIXME:
   For now only two food, two gold one shield and two masks can be drawn per
   unit, the proper way to do this is probably something like what Civ II does.
-  (One food/shield/mask drawn N times, possibly one top of itself. -- SKi 
+  (One food/shield/mask drawn N times, possibly one top of itself. -- SKi
 **************************************************************************/
 void put_unit_image_city_overlays(struct unit *punit, GtkImage *p,
                                   int height,
@@ -522,27 +537,30 @@ void put_unit_image_city_overlays(struct unit *punit, GtkImage *p,
   cairo_surface_destroy(store.surface);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Put overlay tile to pixmap
 **************************************************************************/
 void pixmap_put_overlay_tile(GdkWindow *pixmap, float zoom,
-			     int canvas_x, int canvas_y,
-			     struct sprite *ssprite)
+                             int canvas_x, int canvas_y,
+                             struct sprite *ssprite)
 {
   cairo_t *cr;
+  GdkDrawingContext *ctx;
 
   if (!ssprite) {
     return;
   }
 
-  cr = gdk_cairo_create(pixmap);
+  ctx = gdk_window_begin_draw_frame(pixmap,
+                                    gdk_window_get_clip_region(pixmap));
+  cr = gdk_drawing_context_get_cairo_context(ctx);
   cairo_scale(cr, zoom, zoom);
   cairo_set_source_surface(cr, ssprite->surface, canvas_x, canvas_y);
   cairo_paint(cr);
-  cairo_destroy(cr);
+  gdk_window_end_draw_frame(pixmap, ctx);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Only used for isometric view.
 **************************************************************************/
 void pixmap_put_overlay_tile_draw(struct canvas *pcanvas,
@@ -619,23 +637,23 @@ void pixmap_put_overlay_tile_draw(struct canvas *pcanvas,
   }
 }
 
-/**************************************************************************
- Draws a cross-hair overlay on a tile
+/**********************************************************************//**
+  Draws a cross-hair overlay on a tile
 **************************************************************************/
 void put_cross_overlay_tile(struct tile *ptile)
 {
   float canvas_x, canvas_y;
 
-  if (tile_to_canvas_pos(&canvas_x, &canvas_y, ptile)) {
+  if (tile_to_canvas_pos(&canvas_x, &canvas_y, map_zoom, ptile)) {
     pixmap_put_overlay_tile(gtk_widget_get_window(map_canvas), map_zoom,
 			    canvas_x / map_zoom, canvas_y / map_zoom,
 			    get_attention_crosshair_sprite(tileset));
   }
 }
 
-/*****************************************************************************
- Sets the position of the overview scroll window based on mapview position.
-*****************************************************************************/
+/**********************************************************************//**
+  Sets the position of the overview scroll window based on mapview position.
+**************************************************************************/
 void update_overview_scroll_window_pos(int x, int y)
 {
   gdouble ov_scroll_x, ov_scroll_y;
@@ -657,7 +675,7 @@ void update_overview_scroll_window_pos(int x, int y)
   gtk_adjustment_set_value(ov_vadj, ov_scroll_y);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Refresh map canvas scrollbars
 **************************************************************************/
 void update_map_canvas_scrollbars(void)
@@ -672,7 +690,7 @@ void update_map_canvas_scrollbars(void)
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Refresh map canvas scrollbar as canvas size changes
 **************************************************************************/
 void update_map_canvas_scrollbars_size(void)
@@ -697,7 +715,7 @@ void update_map_canvas_scrollbars_size(void)
 	GINT_TO_POINTER(FALSE));
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Scrollbar has moved
 **************************************************************************/
 void scrollbar_jump_callback(GtkAdjustment *adj, gpointer hscrollbar)
@@ -716,10 +734,10 @@ void scrollbar_jump_callback(GtkAdjustment *adj, gpointer hscrollbar)
     scroll_y = gtk_adjustment_get_value(adj);
   }
 
-  set_mapview_scroll_pos(scroll_x, scroll_y);
+  set_mapview_scroll_pos(scroll_x, scroll_y, mouse_zoom);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Draws a rectangle with top left corner at (canvas_x, canvas_y), and
   width 'w' and height 'h'. It is drawn using the 'selection_gc' context,
   so the pixel combining function is XOR. This means that drawing twice
@@ -733,6 +751,8 @@ void draw_selection_rectangle(int canvas_x, int canvas_y, int w, int h)
   double dashes[2] = {4.0, 4.0};
   struct color *pcolor;
   cairo_t *cr;
+  GdkDrawingContext *ctx;
+  GdkWindow *wndw;
 
   if (w == 0 || h == 0) {
     return;
@@ -743,25 +763,20 @@ void draw_selection_rectangle(int canvas_x, int canvas_y, int w, int h)
     return;
   }
 
-  cr = gdk_cairo_create(gtk_widget_get_window(map_canvas));
+  wndw = gtk_widget_get_window(map_canvas);
+  ctx = gdk_window_begin_draw_frame(wndw,
+                                    gdk_window_get_clip_region(wndw));
+  cr = gdk_drawing_context_get_cairo_context(ctx);
   gdk_cairo_set_source_rgba(cr, &pcolor->color);
   cairo_set_line_width(cr, 2.0);
   cairo_set_dash(cr, dashes, 2, 0);
-#ifdef FREECIV_MSWINDOWS
-  if (cairo_version() < CAIRO_VERSION_ENCODE(1, 12, 0)) {
-    /* Cairo has crashing CAIRO_OPERATOR_DIFFERENCE on win32 surface */
-    cairo_set_operator(cr, CAIRO_OPERATOR_XOR);
-  } else
-#endif /* FREECIV_MSWINDOWS */
-  {
-    cairo_set_operator(cr, CAIRO_OPERATOR_DIFFERENCE);
-  }
+  cairo_set_operator(cr, CAIRO_OPERATOR_DIFFERENCE);
   cairo_rectangle(cr, canvas_x, canvas_y, w, h);
   cairo_stroke(cr);
-  cairo_destroy(cr);
+  gdk_window_end_draw_frame(wndw, ctx);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   This function is called when the tileset is changed.
 **************************************************************************/
 void tileset_changed(void)
@@ -784,7 +799,7 @@ void tileset_changed(void)
 #endif /* FREECIV_MSWINDOWS */
 }
 
-/**************************************************************************
+/**********************************************************************//**
   New turn callback
 **************************************************************************/
 void start_turn(void)
