@@ -35,7 +35,7 @@
 #include "messagewin_g.h"
 #include "options.h"
 
-/* client/gui-gtk-3.22 */
+/* client/gui-gtk-5.0 */
 #include "cityrep.h"
 #include "dialogs.h"
 #include "gui_main.h"
@@ -63,26 +63,94 @@ static struct cma_dialog *get_cma_dialog(struct city *pcity);
 
 static void update_cma_preset_list(struct cma_dialog *pdialog);
 
-static gboolean cma_preset_key_pressed_callback(GtkWidget *w, GdkEventKey *ev,
-						gpointer data);
+static gboolean cma_preset_key_pressed(GtkEventControllerKey *controller,
+                                       guint keyval, guint keycode,
+                                       GdkModifierType state, gpointer data);
 static void cma_del_preset_callback(GtkWidget *w, gpointer data);
 static void cma_preset_remove(struct cma_dialog *pdialog, int preset_index);
 static void cma_preset_remove_response(GtkWidget *w, gint response,
-				       gpointer data);
+                                       gpointer data);
 
 static void cma_add_preset_callback(GtkWidget *w, gpointer data);
 static void cma_preset_add_popup_callback(gpointer data, gint response,
                                           const char *input);
 
 static void cma_active_callback(GtkWidget *w, gpointer data);
-static void cma_activate_preset_callback(GtkTreeView *view, GtkTreePath *path,
-				         GtkTreeViewColumn *col, gpointer data);
+static void cma_activate_preset_callback(GtkSelectionModel *self,
+                                         guint position,
+                                         guint n_items,
+                                         gpointer data);
 
 static void hscale_changed(GtkWidget *get, gpointer data);
 static void set_hscales(const struct cm_parameter *const parameter,
-			struct cma_dialog *pdialog);
+                        struct cma_dialog *pdialog);
 
-/**************************************************************************
+#define FC_TYPE_PRESET_ROW (fc_preset_row_get_type())
+
+G_DECLARE_FINAL_TYPE(FcPresetRow, fc_preset_row, FC, PRESET_ROW, GObject)
+
+struct _FcPresetRow
+{
+  GObject parent_instance;
+
+  char *desc;
+};
+
+struct _FcPresetRowClass
+{
+  GObjectClass parent_class;
+};
+
+G_DEFINE_TYPE(FcPresetRow, fc_preset_row, G_TYPE_OBJECT)
+
+/**********************************************************************//**
+  Finalizing method for FcPresetRow class
+**************************************************************************/
+static void fc_preset_row_finalize(GObject *gobject)
+{
+  FcPresetRow *row = FC_PRESET_ROW(gobject);
+
+  if (row->desc != nullptr) {
+    free(row->desc);
+    row->desc = nullptr;
+  }
+
+  G_OBJECT_CLASS(fc_preset_row_parent_class)->finalize(gobject);
+}
+
+/**********************************************************************//**
+  Initialization method for FcPresetRow class
+**************************************************************************/
+static void
+fc_preset_row_class_init(FcPresetRowClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS(klass);
+
+  object_class->finalize = fc_preset_row_finalize;
+}
+
+/**********************************************************************//**
+  Initialization method for FcPresetRow
+**************************************************************************/
+static void
+fc_preset_row_init(FcPresetRow *self)
+{
+  self->desc = nullptr;
+}
+
+/**********************************************************************//**
+  FcPresetRow creation method
+**************************************************************************/
+static FcPresetRow *fc_preset_row_new(void)
+{
+  FcPresetRow *result;
+
+  result = g_object_new(FC_TYPE_PRESET_ROW, nullptr);
+
+  return result;
+}
+
+/**********************************************************************//**
   Initialize cma front end system
 **************************************************************************/
 void cma_fe_init(void)
@@ -90,7 +158,7 @@ void cma_fe_init(void)
   dialog_list = dialog_list_new();
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Free resources allocated for cma front end system
 **************************************************************************/
 void cma_fe_done(void)
@@ -98,7 +166,7 @@ void cma_fe_done(void)
   dialog_list_destroy(dialog_list);
 }
 
-/**************************************************************************
+/**********************************************************************//**
  only called when the city dialog is closed.
 **************************************************************************/
 void close_cma_dialog(struct city *pcity)
@@ -109,10 +177,12 @@ void close_cma_dialog(struct city *pcity)
     /* A city which is being investigated doesn't contain cma dialog */
     return;
   }
-  gtk_widget_destroy(pdialog->shell);
+
+  gtk_box_remove(GTK_BOX(gtk_widget_get_parent(pdialog->shell)),
+                 pdialog->shell);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Destroy cma dialog
 **************************************************************************/
 static void cma_dialog_destroy_callback(GtkWidget *w, gpointer data)
@@ -123,9 +193,9 @@ static void cma_dialog_destroy_callback(GtkWidget *w, gpointer data)
   free(pdialog);
 }
 
-/****************************************************************
- return the cma_dialog for a given city.
-*****************************************************************/
+/**********************************************************************//**
+  Return the cma_dialog for a given city.
+**************************************************************************/
 struct cma_dialog *get_cma_dialog(struct city *pcity)
 {
   dialog_list_iterate(dialog_list, pdialog) {
@@ -137,21 +207,32 @@ struct cma_dialog *get_cma_dialog(struct city *pcity)
   return NULL;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   User has pressed button in cma dialog
 **************************************************************************/
-static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev,
-				      gpointer data)
+static gboolean button_press_callback(GtkGestureClick *gesture, int n_press,
+                                      double x, double y)
 {
-  GtkTreePath *path;
-  GtkTreeViewColumn *column;
+  GtkWidget *widget = gtk_event_controller_get_widget(GTK_EVENT_CONTROLLER(gesture));
+  struct cma_dialog *pdialog
+    = (struct cma_dialog *) g_object_get_data(G_OBJECT(widget), "dialog");
+  int rnbr = get_column_view_row(pdialog->preset_list, y);
 
-  if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(view),
-	ev->x, ev->y, &path, &column, NULL, NULL)) {
-    if (ev->type == GDK_BUTTON_PRESS) {
-      cma_activate_preset_callback(view, path, column, data);
-    } else if (ev->type == GDK_2BUTTON_PRESS) {
-      struct cma_dialog *pdialog = (struct cma_dialog *) data;
+  if (rnbr >= 0) {
+    if (n_press == 1) {
+      const struct cm_parameter *pparam;
+
+      pparam = cmafec_preset_get_parameter(rnbr);
+
+      /* Save the change */
+      cmafec_set_fe_parameter(pdialog->pcity, pparam);
+
+      if (cma_is_city_under_agent(pdialog->pcity, NULL)) {
+        cma_release_city(pdialog->pcity);
+        cma_put_city_under_agent(pdialog->pcity, pparam);
+      }
+      refresh_city_dialog(pdialog->pcity);
+    } else if (n_press == 2) {
       struct cm_parameter param;
 
       cmafec_get_fe_parameter(pdialog->pcity, &param);
@@ -159,12 +240,11 @@ static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev,
       refresh_city_dialog(pdialog->pcity);
     }
   }
-  gtk_tree_path_free(path);
 
   return FALSE;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   User has requested help
 **************************************************************************/
 static void help_callback(GtkWidget *w, gpointer data)
@@ -172,56 +252,47 @@ static void help_callback(GtkWidget *w, gpointer data)
   popup_help_dialog_string(HELP_CMA_ITEM);
 }
 
-/**************************************************************************
-  Cell data function for cma dialog 
+/**********************************************************************//**
+  Wlmeta table cell bind function
 **************************************************************************/
-static void cell_data_func(GtkTreeViewColumn *col, GtkCellRenderer *cell,
-                           GtkTreeModel *model, GtkTreeIter *it, gpointer data)
+static void preset_factory_bind(GtkSignalListItemFactory *self,
+                                GtkListItem *list_item,
+                                gpointer user_data)
 {
-  struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  char *s1;
-  const char *s2;
-  int i1, i2;
-  struct cm_parameter param;
-  GtkTreePath *path;
+  FcPresetRow *row;
+  GtkWidget *child = gtk_list_item_get_child(list_item);
 
-  gtk_tree_model_get(model, it, 0, &s1, -1);
-  if (s1 == NULL) {
-    return;
-  }
-  path = gtk_tree_model_get_path(model, it);
-  i1 = gtk_tree_path_get_indices(path)[0];
-  gtk_tree_path_free(path);
+  row = gtk_list_item_get_item(list_item);
 
-  cmafec_get_fe_parameter(pdialog->pcity, &param);
-  s2 = cmafec_get_short_descr(&param);
-  i2 = cmafec_preset_get_index_of_parameter(&param);
-
-  if (!strcmp(s1, s2) && i1 == i2) {
-    g_object_set(G_OBJECT(cell), "style", PANGO_STYLE_ITALIC,
-                 "weight", PANGO_WEIGHT_BOLD, NULL);
-  } else {
-    g_object_set(G_OBJECT(cell), "style", PANGO_STYLE_NORMAL,
-                 "weight", PANGO_WEIGHT_NORMAL, NULL);
-  }
-
-  g_free(s1);
+  gtk_label_set_text(GTK_LABEL(child), row->desc);
 }
 
-/**************************************************************************
+/**********************************************************************//**
+  Wlmeta table cell setup function
+**************************************************************************/
+static void preset_factory_setup(GtkSignalListItemFactory *self,
+                                 GtkListItem *list_item,
+                                 gpointer user_data)
+{
+  gtk_list_item_set_child(list_item, gtk_label_new(""));
+}
+
+/**********************************************************************//**
   Instantiates a new struct for each city_dialog window that is open.
 **************************************************************************/
 struct cma_dialog *create_cma_dialog(struct city *pcity, bool tiny)
 {
   struct cma_dialog *pdialog;
   struct cm_parameter param;
-  GtkWidget *frame, *page, *hbox, *label, *table;
-  GtkWidget *vbox, *sw, *hscale, *button;
-  GtkListStore *store;
-  GtkCellRenderer *rend;
-  GtkWidget *view;
-  GtkTreeViewColumn *column;
+  GtkWidget *frame, *page, *hbox, *vbox, *label, *table;
+  GtkWidget *hscale, *button;
+  GtkListItemFactory *factory;
+  GtkWidget *list;
+  GtkColumnViewColumn *column;
   gint layout_width;
+  GtkEventController *controller;
+  int shell_row = 0;
+  PangoLayout *layout;
 
   cmafec_get_fe_parameter(pcity, &param);
   pdialog = fc_malloc(sizeof(struct cma_dialog));
@@ -230,113 +301,108 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, bool tiny)
   gtk_orientable_set_orientation(GTK_ORIENTABLE(pdialog->shell),
                                  GTK_ORIENTATION_VERTICAL);
   gtk_grid_set_row_spacing(GTK_GRID(pdialog->shell), 8);
-  gtk_container_set_border_width(GTK_CONTAINER(pdialog->shell), 8);
+  gtk_widget_set_margin_start(pdialog->shell, 8);
+  gtk_widget_set_margin_end(pdialog->shell, 8);
+  gtk_widget_set_margin_top(pdialog->shell, 8);
+  gtk_widget_set_margin_bottom(pdialog->shell, 8);
   g_signal_connect(pdialog->shell, "destroy",
                    G_CALLBACK(cma_dialog_destroy_callback), pdialog);
 
-  page = gtk_grid_new();
-  gtk_grid_set_column_spacing(GTK_GRID(page), 12);
-  gtk_container_add(GTK_CONTAINER(pdialog->shell), page);
+  page = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
+  gtk_grid_attach(GTK_GRID(pdialog->shell), page, 0, shell_row++, 1, 1);
 
-  vbox = gtk_grid_new();
-  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
-                                 GTK_ORIENTATION_VERTICAL);
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
   gtk_grid_set_row_spacing(GTK_GRID(pdialog->shell), 2);
-  gtk_container_add(GTK_CONTAINER(page), vbox);
+  gtk_box_append(GTK_BOX(page), vbox);
 
-  sw = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
-                                      GTK_SHADOW_ETCHED_IN);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
-                                 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  pdialog->store = g_list_store_new(FC_TYPE_PRESET_ROW);
+  pdialog->selection
+    = gtk_single_selection_new(G_LIST_MODEL(pdialog->store));
+  list = gtk_column_view_new(GTK_SELECTION_MODEL(pdialog->selection));
+  pdialog->preset_list = list;
 
-  store = gtk_list_store_new(1, G_TYPE_STRING);
-  pdialog->store = store;
+  factory = gtk_signal_list_item_factory_new();
+  g_signal_connect(factory, "bind", G_CALLBACK(preset_factory_bind),
+                   nullptr);
+  g_signal_connect(factory, "setup", G_CALLBACK(preset_factory_setup),
+                   nullptr);
 
-  view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
-  gtk_widget_set_hexpand(view, TRUE);
-  gtk_widget_set_vexpand(view, TRUE);
-  g_object_unref(store);
-  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
-  pdialog->preset_list = view;
-  pdialog->selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
+  column = gtk_column_view_column_new(_("Name"), factory);
+  gtk_column_view_append_column(GTK_COLUMN_VIEW(list), column);
 
-  g_signal_connect(pdialog->preset_list, "button_press_event",
-                   G_CALLBACK(button_press_callback), pdialog);
+  g_object_set_data(G_OBJECT(pdialog->preset_list), "dialog", pdialog);
+  controller = GTK_EVENT_CONTROLLER(gtk_gesture_click_new());
+  g_signal_connect(controller, "pressed",
+                   G_CALLBACK(button_press_callback), NULL);
+  gtk_widget_add_controller(pdialog->preset_list, controller);
 
-  gtk_widget_set_tooltip_text(view,
+  gtk_widget_set_tooltip_text(list,
                               _("For information on\n"
                                 "the citizen governor and governor presets,\n"
                                 "including sample presets,\n"
                                 "see README.governor."));
 
-  rend = gtk_cell_renderer_text_new();
-  column = gtk_tree_view_column_new_with_attributes(NULL, rend,
-                                                    "text", 0, NULL);
-  gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
-  gtk_tree_view_column_set_cell_data_func(column, rend, cell_data_func,
-                                          pdialog, NULL);
-
   label = g_object_new(GTK_TYPE_LABEL,
                        "use-underline", TRUE,
-                       "mnemonic-widget", view,
+                       "mnemonic-widget", list,
                        "label", _("Prese_ts:"),
                        "xalign", 0.0, "yalign", 0.5, NULL);
-  gtk_container_add(GTK_CONTAINER(vbox), label);
+  gtk_box_append(GTK_BOX(vbox), label);
+  gtk_box_append(GTK_BOX(vbox), list);
 
-  gtk_container_add(GTK_CONTAINER(sw), view);
-  gtk_container_add(GTK_CONTAINER(vbox), sw);
-
-  g_signal_connect(view, "row_activated",
+  g_signal_connect(pdialog->selection, "selection-changed",
                    G_CALLBACK(cma_activate_preset_callback), pdialog);
-  g_signal_connect(view, "key-press-event",
-                   G_CALLBACK(cma_preset_key_pressed_callback), pdialog);
+  controller = gtk_event_controller_key_new();
+  g_signal_connect(controller, "key-pressed",
+                   G_CALLBACK(cma_preset_key_pressed), pdialog);
+  gtk_widget_add_controller(list, controller);
 
-  hbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
-  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_EDGE);
-  gtk_container_add(GTK_CONTAINER(vbox), hbox);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
+  gtk_box_append(GTK_BOX(vbox), hbox);
 
   button = icon_label_button_new("document-new", _("Ne_w"));
-  gtk_container_add(GTK_CONTAINER(hbox), button);
+  gtk_box_append(GTK_BOX(hbox), button);
   g_signal_connect(button, "clicked",
                    G_CALLBACK(cma_add_preset_callback), pdialog);
   pdialog->add_preset_command = button;
 
   pdialog->del_preset_command = icon_label_button_new("edit-delete",
                                                       _("_Delete"));
-  gtk_container_add(GTK_CONTAINER(hbox), pdialog->del_preset_command);
+  gtk_box_append(GTK_BOX(hbox), pdialog->del_preset_command);
   g_signal_connect(pdialog->del_preset_command, "clicked",
                    G_CALLBACK(cma_del_preset_callback), pdialog);
 
-  /* the right-hand side */
-
-  vbox = gtk_grid_new();
-  g_object_set(vbox, "margin", 2, NULL);
-  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
-                                 GTK_ORIENTATION_VERTICAL);
-  gtk_container_add(GTK_CONTAINER(page), vbox);
+  /* The right-hand side */
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
+  gtk_widget_set_margin_bottom(vbox, 2);
+  gtk_widget_set_margin_end(vbox, 2);
+  gtk_widget_set_margin_start(vbox, 2);
+  gtk_widget_set_margin_top(vbox, 2);
+  gtk_box_append(GTK_BOX(page), vbox);
 
   /* Result */
   if (!tiny) {
     frame = gtk_frame_new(_("Results"));
     gtk_widget_set_vexpand(frame, TRUE);
     gtk_widget_set_valign(frame, GTK_ALIGN_CENTER);
-    gtk_container_add(GTK_CONTAINER(vbox), frame);
+    gtk_box_append(GTK_BOX(vbox), frame);
 
     pdialog->result_label =
       gtk_label_new("food\n prod\n trade\n\n people\n grow\n prod\n name");
     gtk_widget_set_name(pdialog->result_label, "city_label");
-    gtk_container_add(GTK_CONTAINER(frame), pdialog->result_label);
+    gtk_frame_set_child(GTK_FRAME(frame), pdialog->result_label);
     gtk_label_set_justify(GTK_LABEL(pdialog->result_label), GTK_JUSTIFY_LEFT);
   } else {
     pdialog->result_label = NULL;
   }
 
   /* Minimal Surplus and Factor */
-
   table = gtk_grid_new();
-  g_object_set(table, "margin", 2, NULL);
-  gtk_container_add(GTK_CONTAINER(vbox), table);
+  gtk_widget_set_margin_bottom(table, 2);
+  gtk_widget_set_margin_end(table, 2);
+  gtk_widget_set_margin_start(table, 2);
+  gtk_widget_set_margin_top(table, 2);
+  gtk_box_append(GTK_BOX(vbox), table);
 
   label = gtk_label_new(_("Minimal Surplus"));
   gtk_widget_set_halign(label, GTK_ALIGN_START);
@@ -359,9 +425,11 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, bool tiny)
                         GUI_GTK_OPTION(governor_range_min),
                         GUI_GTK_OPTION(governor_range_max));
     gtk_range_set_increments(GTK_RANGE(hscale), 1, 1);
-    pango_layout_get_pixel_size(gtk_scale_get_layout(GTK_SCALE(hscale)),
-                                &layout_width, NULL);
-    gtk_widget_set_size_request(hscale, layout_width + 51 * 2, -1);
+    layout = gtk_scale_get_layout(GTK_SCALE(hscale));
+    if (layout != NULL) {
+      pango_layout_get_pixel_size(layout, &layout_width, NULL);
+      gtk_widget_set_size_request(hscale, layout_width + 51 * 2, -1);
+    }
 
     gtk_grid_attach(GTK_GRID(table), hscale, 1, i + 1, 1, 1);
     gtk_scale_set_digits(GTK_SCALE(hscale), 0);
@@ -371,12 +439,16 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, bool tiny)
                      "value-changed",
                      G_CALLBACK(hscale_changed), pdialog);
 
-    pdialog->factor[i] = hscale =
-        gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, NULL);
+    pdialog->factor[i] = hscale
+      = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, NULL);
     gtk_range_set_range(GTK_RANGE(hscale), 0, 25);
     gtk_range_set_increments(GTK_RANGE(hscale), 1, 1);
-    pango_layout_get_pixel_size(gtk_scale_get_layout(GTK_SCALE(hscale)),
-                                &layout_width, NULL);
+    layout = gtk_scale_get_layout(GTK_SCALE(hscale));
+    if (layout != NULL) {
+      pango_layout_get_pixel_size(layout, &layout_width, NULL);
+    } else {
+      layout_width = 20;
+    }
     gtk_widget_set_size_request(hscale, layout_width + 26 * 2, -1);
 
     gtk_grid_attach(GTK_GRID(table), hscale, 2, i + 1, 1, 1);
@@ -388,7 +460,6 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, bool tiny)
   } output_type_iterate_end;
 
   /* Happy Surplus and Factor */
-
   label = gtk_label_new(_("Celebrate"));
   gtk_grid_attach(GTK_GRID(table), label, 0, O_LAST + 1, 1, 1);
   gtk_widget_set_halign(label, GTK_ALIGN_START);
@@ -396,20 +467,22 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, bool tiny)
 
   pdialog->happy_button = gtk_check_button_new();
   gtk_widget_set_halign(pdialog->happy_button, GTK_ALIGN_END);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdialog->happy_button),
-                               FALSE);
+  gtk_check_button_set_active(GTK_CHECK_BUTTON(pdialog->happy_button),
+                              FALSE);
   gtk_grid_attach(GTK_GRID(table), pdialog->happy_button, 1, O_LAST + 1, 1, 1);
 
   g_signal_connect(pdialog->happy_button, "toggled",
                    G_CALLBACK(hscale_changed), pdialog);
 
-  pdialog->factor[O_LAST] = hscale =
-      gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, NULL);
+  pdialog->factor[O_LAST] = hscale
+    = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, NULL);
   gtk_range_set_range(GTK_RANGE(hscale), 0, 50);
   gtk_range_set_increments(GTK_RANGE(hscale), 1, 1);
-  pango_layout_get_pixel_size(gtk_scale_get_layout(GTK_SCALE(hscale)),
-                              &layout_width, NULL);
-  gtk_widget_set_size_request(hscale, layout_width + 51 * 2, -1);
+  layout = gtk_scale_get_layout(GTK_SCALE(hscale));
+  if (layout != NULL) {
+    pango_layout_get_pixel_size(layout, &layout_width, NULL);
+    gtk_widget_set_size_request(hscale, layout_width + 51 * 2, -1);
+  }
 
   gtk_grid_attach(GTK_GRID(table), hscale, 2, O_LAST + 1, 1, 1);
   gtk_scale_set_digits(GTK_SCALE(hscale), 0);
@@ -419,41 +492,48 @@ struct cma_dialog *create_cma_dialog(struct city *pcity, bool tiny)
                    "value-changed",
                    G_CALLBACK(hscale_changed), pdialog);
 
-  /* buttons */
+  /* Maximize Growth */
+  label = gtk_label_new(_("Maximize growth"));
+  gtk_grid_attach(GTK_GRID(table), label, 0, O_LAST + 2, 1, 1);
+  gtk_widget_set_halign(label, GTK_ALIGN_START);
+  gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
 
-  hbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
-  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_EDGE);
-  gtk_container_add(GTK_CONTAINER(vbox), hbox);
+  pdialog->growth_button = gtk_check_button_new();
+  gtk_widget_set_halign(pdialog->growth_button, GTK_ALIGN_END);
+  gtk_check_button_set_active(GTK_CHECK_BUTTON(pdialog->growth_button),
+                              FALSE);
+  gtk_grid_attach(GTK_GRID(table), pdialog->growth_button, 1, O_LAST + 2, 1, 1);
+
+  g_signal_connect(pdialog->growth_button, "toggled",
+                   G_CALLBACK(hscale_changed), pdialog);
+
+  /* Buttons */
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
+  gtk_box_append(GTK_BOX(vbox), hbox);
 
   button = icon_label_button_new("help-browser", _("Help"));
   g_signal_connect(button, "clicked",
                    G_CALLBACK(help_callback), NULL);
-  gtk_container_add(GTK_CONTAINER(hbox), button);
-  gtk_button_box_set_child_non_homogeneous(GTK_BUTTON_BOX(hbox),
-                                           button, TRUE);
+  gtk_box_append(GTK_BOX(hbox), button);
 
   pdialog->active_command = gtk_toggle_button_new();
   gtk_button_set_use_underline(GTK_BUTTON(pdialog->active_command), TRUE);
-  gtk_button_set_image_position(GTK_BUTTON(pdialog->active_command),
-                                GTK_POS_TOP);
   gtk_widget_set_name(pdialog->active_command, "comment_label");
-  gtk_container_add(GTK_CONTAINER(hbox), pdialog->active_command);
+  gtk_box_append(GTK_BOX(hbox), pdialog->active_command);
 
-  gtk_widget_show_all(pdialog->shell);
+  gtk_widget_set_visible(pdialog->shell, TRUE);
 
   dialog_list_prepend(dialog_list, pdialog);
 
   update_cma_preset_list(pdialog);
 
-  gtk_tree_view_focus(GTK_TREE_VIEW(view));
-
-  /* refresh is done in refresh_city_dialog */
+  /* Refresh is done in refresh_city_dialog() */
 
   return pdialog;
 }
 
-/**************************************************************************
- refreshes the cma dialog
+/**********************************************************************//**
+  Refreshes the cma dialog
 **************************************************************************/
 void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
 {
@@ -465,13 +545,13 @@ void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
   cmafec_get_fe_parameter(pcity, &param);
 
   if (pdialog->result_label != NULL) {
-    /* fill in result label */
+    /* Fill in result label */
     cm_result_from_main_map(result, pcity);
     gtk_label_set_text(GTK_LABEL(pdialog->result_label),
                        cmafec_get_result_descr(pcity, result, &param));
   }
 
-  /* if called from a hscale, we _don't_ want to do this */
+  /* If called from a hscale, we _don't_ want to do this */
   if (refresh != DONT_REFRESH_HSCALES) {
     set_hscales(&param, pdialog);
   }
@@ -503,43 +583,45 @@ void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
   cm_result_destroy(result);
 }
 
-/**************************************************************************
- fills in the preset list
+/**********************************************************************//**
+  Fills in the preset list
 **************************************************************************/
 static void update_cma_preset_list(struct cma_dialog *pdialog)
 {
   char buf[BUFFER_SIZE];
-  GtkTreeIter it;
   int i;
 
   /* Fill preset list */
-  gtk_list_store_clear(pdialog->store);
+  g_list_store_remove_all(pdialog->store);
 
   /* Append the presets */
   if (cmafec_preset_num()) {
     for (i = 0; i < cmafec_preset_num(); i++) {
+      FcPresetRow *row = fc_preset_row_new();
+
       fc_strlcpy(buf, cmafec_preset_get_descr(i), sizeof(buf));
-      gtk_list_store_append(pdialog->store, &it);
-      gtk_list_store_set(pdialog->store, &it, 0, buf, -1);
+      row->desc = fc_strdup(buf);
+      g_list_store_append(pdialog->store, row);
+      g_object_unref(row);
     }
   }
 }
 
-/****************************************************************
- callback for selecting a preset from the preset view
-*****************************************************************/
-static void cma_activate_preset_callback(GtkTreeView *view, GtkTreePath *path,
-				         GtkTreeViewColumn *col, gpointer data)
+/**********************************************************************//**
+  Callback for selecting a preset from the preset view
+**************************************************************************/
+static void cma_activate_preset_callback(GtkSelectionModel *self,
+                                         guint position,
+                                         guint n_items,
+                                         gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
-  int preset_index;
   const struct cm_parameter *pparam;
-
-  preset_index = gtk_tree_path_get_indices(path) [0];
+  int preset_index = gtk_single_selection_get_selected(GTK_SINGLE_SELECTION(self));
 
   pparam = cmafec_preset_get_parameter(preset_index);
 
-  /* save the change */
+  /* Save the change */
   cmafec_set_fe_parameter(pdialog->pcity, pparam);
 
   if (cma_is_city_under_agent(pdialog->pcity, NULL)) {
@@ -549,17 +631,17 @@ static void cma_activate_preset_callback(GtkTreeView *view, GtkTreePath *path,
   refresh_city_dialog(pdialog->pcity);
 }
 
-/**************************************************************************
- pops up a dialog to allow to name your new preset
+/**********************************************************************//**
+  Pops up a dialog to allow to name your new preset
 **************************************************************************/
 static void cma_add_preset_callback(GtkWidget *w, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
   const char *default_name;
-  GtkWidget *parent = gtk_widget_get_toplevel(pdialog->shell);
+  GtkWidget *parent = gtk_widget_get_ancestor(pdialog->shell, GTK_TYPE_WINDOW);
   int index;
 
-  if ((index = gtk_tree_selection_get_row(pdialog->selection)) != -1) {
+  if ((index = gtk_single_selection_get_selected(pdialog->selection)) >= 0) {
     default_name = cmafec_preset_get_descr(index);
   } else {
     default_name = _("new preset");
@@ -572,9 +654,9 @@ static void cma_add_preset_callback(GtkWidget *w, gpointer data)
                                     cma_preset_add_popup_callback, pdialog);
 }
 
-/****************************************************************
-  callback for the add_preset popup
-*****************************************************************/
+/**********************************************************************//**
+  Callback for the add_preset popup
+**************************************************************************/
 static void cma_preset_add_popup_callback(gpointer data, gint response,
                                           const char *input)
 {
@@ -588,7 +670,7 @@ static void cma_preset_add_popup_callback(gpointer data, gint response,
       cmafec_preset_add(input, &param);
       update_cma_preset_list(pdialog);
       refresh_cma_dialog(pdialog->pcity, DONT_REFRESH_HSCALES);
-      /* if this or other cities have this set as "custom" */
+      /* If this or other cities have this set as "custom" */
       city_report_dialog_update();
     } /* else CANCEL or DELETE_EVENT */
 
@@ -596,81 +678,79 @@ static void cma_preset_add_popup_callback(gpointer data, gint response,
   }
 }
 
-/****************************************************************
+/**********************************************************************//**
   Key pressed in preset list
-*****************************************************************/
-static gboolean cma_preset_key_pressed_callback(GtkWidget *w, GdkEventKey *ev,
-						gpointer data)
+**************************************************************************/
+static gboolean cma_preset_key_pressed(GtkEventControllerKey *controller,
+                                       guint keyval, guint keycode,
+                                       GdkModifierType state, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
   int index;
 
-  if ((index = gtk_tree_selection_get_row(pdialog->selection)) == -1) {
+  if ((index = gtk_single_selection_get_selected(pdialog->selection)) < 0) {
     return FALSE;
   }
 
-  if (ev->type == GDK_KEY_PRESS) {
-    switch (ev->keyval) {
-    case GDK_KEY_Delete:
-      cma_preset_remove(pdialog, index);
-      break;
-    case GDK_KEY_Insert:
-      cma_add_preset_callback(NULL, pdialog);
-      break;
-    default:
-      return FALSE;
-    }
-    return TRUE;
+  switch (keyval) {
+  case GDK_KEY_Delete:
+    cma_preset_remove(pdialog, index);
+    break;
+  case GDK_KEY_Insert:
+    cma_add_preset_callback(NULL, pdialog);
+    break;
+  default:
+    return FALSE;
   }
-  return FALSE;
+
+  return TRUE;
 }
 
-
-/**************************************************************************
- callback for del_preset 
+/**********************************************************************//**
+  Callback for del_preset
 **************************************************************************/
 static void cma_del_preset_callback(GtkWidget *w, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
   int index;
 
-  if ((index = gtk_tree_selection_get_row(pdialog->selection)) == -1) {
+  if ((index = gtk_single_selection_get_selected(pdialog->selection)) < 0) {
     return;
   }
 
   cma_preset_remove(pdialog, index);
 }
 
-/**************************************************************************
- pops up a dialog to remove a preset
+/**********************************************************************//**
+  Pops up a dialog to remove a preset
 **************************************************************************/
 static void cma_preset_remove(struct cma_dialog *pdialog, int preset_index)
 {
-  GtkWidget *parent = gtk_widget_get_toplevel(pdialog->shell), *shl;
+  GtkWidget *parent = gtk_widget_get_ancestor(pdialog->shell, GTK_TYPE_WINDOW);
+  GtkWidget *shl;
 
   pdialog->id = preset_index;
   shl = gtk_message_dialog_new(NULL,
-			       GTK_DIALOG_DESTROY_WITH_PARENT,
-			       GTK_MESSAGE_QUESTION,
-			       GTK_BUTTONS_YES_NO,
-			       _("Remove this preset?"));
+                               GTK_DIALOG_DESTROY_WITH_PARENT,
+                               GTK_MESSAGE_QUESTION,
+                               GTK_BUTTONS_YES_NO,
+                               _("Remove this preset?"));
   setup_dialog(shl, parent);
   pdialog->preset_remove_shell = shl;
 
   gtk_window_set_title(GTK_WINDOW(shl), cmafec_preset_get_descr(preset_index));
-  gtk_window_set_position(GTK_WINDOW(shl), GTK_WIN_POS_CENTER_ON_PARENT);
 
   g_signal_connect(shl, "response",
-		   G_CALLBACK(cma_preset_remove_response), pdialog);
+                   G_CALLBACK(cma_preset_remove_response), pdialog);
 
   gtk_window_present(GTK_WINDOW(shl));
 }
 
-/****************************************************************
- callback for the remove_preset popup
-*****************************************************************/
+/**********************************************************************//**
+  Callback for the remove_preset popup
+**************************************************************************/
 static void cma_preset_remove_response(GtkWidget *w, gint response,
-				       gpointer data)
+                                       gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
 
@@ -682,13 +762,13 @@ static void cma_preset_remove_response(GtkWidget *w, gint response,
     /* if this or other cities have this set, reset to "custom" */
     city_report_dialog_update();
   }
-  gtk_widget_destroy(w);
-  
+  gtk_window_destroy(GTK_WINDOW(w));
+
   pdialog->preset_remove_shell = NULL;
 }
 
-/**************************************************************************
- activates/deactivates agent control.
+/**********************************************************************//**
+  Activates/deactivates agent control.
 **************************************************************************/
 static void cma_active_callback(GtkWidget *w, gpointer data)
 {
@@ -705,12 +785,12 @@ static void cma_active_callback(GtkWidget *w, gpointer data)
   refresh_city_dialog(pdialog->pcity);
 }
 
-/****************************************************************
- called to adjust the sliders when a preset is selected
- notice that we don't want to call update_result here. 
-*****************************************************************/
+/**********************************************************************//**
+  Called to adjust the sliders when a preset is selected
+  notice that we don't want to call update_result here.
+**************************************************************************/
 static void set_hscales(const struct cm_parameter *const parameter,
-			struct cma_dialog *pdialog)
+                        struct cma_dialog *pdialog)
 {
   allow_refreshes = 0;
   output_type_iterate(i) {
@@ -718,16 +798,18 @@ static void set_hscales(const struct cm_parameter *const parameter,
                         parameter->minimal_surplus[i]);
     gtk_range_set_value(GTK_RANGE(pdialog->factor[i]), parameter->factor[i]);
   } output_type_iterate_end;
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pdialog->happy_button),
-			       parameter->require_happy);
+  gtk_check_button_set_active(GTK_CHECK_BUTTON(pdialog->happy_button),
+                              parameter->require_happy);
+  gtk_check_button_set_active(GTK_CHECK_BUTTON(pdialog->growth_button),
+                              parameter->max_growth);
   gtk_range_set_value(GTK_RANGE(pdialog->factor[O_LAST]),
                       parameter->happy_factor);
   allow_refreshes = 1;
 }
 
-/************************************************************************
- callback if we moved the sliders.
-*************************************************************************/
+/**********************************************************************//**
+  Callback if we moved the sliders.
+**************************************************************************/
 static void hscale_changed(GtkWidget *get, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
@@ -745,14 +827,16 @@ static void hscale_changed(GtkWidget *get, gpointer data)
         (int) (gtk_range_get_value(GTK_RANGE(pdialog->factor[i])));
   } output_type_iterate_end;
   param.require_happy =
-      (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pdialog->happy_button)) ? 1 : 0);
+      (gtk_check_button_get_active(GTK_CHECK_BUTTON(pdialog->happy_button)) ? TRUE : FALSE);
+  param.max_growth =
+      (gtk_check_button_get_active(GTK_CHECK_BUTTON(pdialog->growth_button)) ? TRUE : FALSE);
   param.happy_factor =
       (int) (gtk_range_get_value(GTK_RANGE(pdialog->factor[O_LAST])));
 
-  /* save the change */
+  /* Save the change */
   cmafec_set_fe_parameter(pdialog->pcity, &param);
 
-  /* refreshes the cma */
+  /* Refreshes the cma */
   if (cma_is_city_under_agent(pdialog->pcity, NULL)) {
     cma_release_city(pdialog->pcity);
     cma_put_city_under_agent(pdialog->pcity, &param);
