@@ -43,7 +43,7 @@
 #include "options.h"
 #include "text.h"
 
-/* clien/gui-gtk-2.0 */
+/* clien/gui-gtk-5.0 */
 #include "plrdlg.h"
 #include "dialogs.h"
 #include "gui_main.h"
@@ -60,8 +60,9 @@ static GtkWidget *all_toggle;
 static GtkListStore *goto_list_store;
 static GtkTreeSelection *goto_list_selection;
 struct tile *original_tile;
+static bool gotodlg_updating = FALSE;
 
-static void update_goto_dialog(GtkToggleButton *button);
+static void update_goto_dialog(GtkCheckButton *button);
 static void update_source_label(void);
 static void refresh_airlift_column(void);
 static void refresh_airlift_button(void);
@@ -83,7 +84,7 @@ enum {
   GD_COL_NUM
 };
 
-/**************************************************************************
+/**********************************************************************//**
   User has responded to goto dialog
 **************************************************************************/
 static void goto_cmd_callback(GtkWidget *dlg, gint arg)
@@ -99,7 +100,7 @@ static void goto_cmd_callback(GtkWidget *dlg, gint arg)
 
       if (pdestcity) {
         unit_list_iterate(get_units_in_focus(), punit) {
-          if (unit_can_airlift_to(punit, pdestcity)) {
+          if (unit_can_airlift_to(&(wld.map), punit, pdestcity)) {
             request_unit_airlift(punit, pdestcity);
           }
         } unit_list_iterate_end;
@@ -112,7 +113,7 @@ static void goto_cmd_callback(GtkWidget *dlg, gint arg)
       struct city *pdestcity = get_selected_city();
 
       if (pdestcity) {
-	unit_list_iterate(get_units_in_focus(), punit) {
+        unit_list_iterate(get_units_in_focus(), punit) {
           send_goto_tile(punit, pdestcity->tile);
         } unit_list_iterate_end;
       }
@@ -123,12 +124,11 @@ static void goto_cmd_callback(GtkWidget *dlg, gint arg)
     break;
   }
 
-  gtk_widget_destroy(dlg);
+  gtk_window_destroy(GTK_WINDOW(dlg));
   dshell = NULL;
 }
 
-
-/**************************************************************************
+/**********************************************************************//**
   Create goto -dialog for gotoing or airlifting unit
 **************************************************************************/
 static void create_goto_dialog(void)
@@ -138,28 +138,27 @@ static void create_goto_dialog(void)
   GtkTreeViewColumn *col;
 
   dshell = gtk_dialog_new_with_buttons(_("Goto/Airlift Unit"),
-    NULL,
-    0,
-    GTK_STOCK_CANCEL,
-    GTK_RESPONSE_CANCEL,
-    _("Air_lift"),
-    CMD_AIRLIFT,
-    _("_Goto"),
-    CMD_GOTO,
-    NULL);
+                                       NULL,
+                                       0,
+                                       _("_Cancel"),
+                                       GTK_RESPONSE_CANCEL,
+                                       _("Air_lift"),
+                                       CMD_AIRLIFT,
+                                       _("_Goto"),
+                                       CMD_GOTO,
+                                       NULL);
   setup_dialog(dshell, toplevel);
-  gtk_window_set_position(GTK_WINDOW(dshell), GTK_WIN_POS_MOUSE);
   gtk_dialog_set_default_response(GTK_DIALOG(dshell), CMD_GOTO);
   g_signal_connect(dshell, "destroy",
-		   G_CALLBACK(gtk_widget_destroyed), &dshell);
+                   G_CALLBACK(widget_destroyed), &dshell);
   g_signal_connect(dshell, "response",
                    G_CALLBACK(goto_cmd_callback), NULL);
 
-  source = gtk_label_new("" /* filled in later */);
-  gtk_label_set_line_wrap(GTK_LABEL(source), TRUE);
+  source = gtk_label_new("" /* Filled in later */);
+  gtk_label_set_wrap(GTK_LABEL(source), TRUE);
   gtk_label_set_justify(GTK_LABEL(source), GTK_JUSTIFY_CENTER);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dshell)->vbox),
-        source, FALSE, FALSE, 0);
+  gtk_box_insert_child_after(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dshell))),
+                             source, NULL);
 
   label = g_object_new(GTK_TYPE_LABEL,
     "use-underline", TRUE,
@@ -169,18 +168,21 @@ static void create_goto_dialog(void)
     NULL);
   frame = gtk_frame_new("");
   gtk_frame_set_label_widget(GTK_FRAME(frame), label);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dshell)->vbox),
-        frame, TRUE, TRUE, 0);
+  gtk_box_insert_child_after(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dshell))),
+                             frame, NULL);
 
-  vbox = gtk_vbox_new(FALSE, 6);
-  gtk_container_add(GTK_CONTAINER(frame), vbox);
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
+  gtk_frame_set_child(GTK_FRAME(frame), vbox);
 
   goto_list_store = gtk_list_store_new(GD_COL_NUM, G_TYPE_INT, G_TYPE_STRING,
-                                       GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
+                                       GDK_TYPE_PIXBUF, G_TYPE_STRING,
+                                       G_TYPE_STRING);
   gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(goto_list_store),
                                        GD_COL_CITY_NAME, GTK_SORT_ASCENDING);
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(goto_list_store));
+  gtk_widget_set_hexpand(view, TRUE);
+  gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(goto_list_store);
   goto_list_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), TRUE);
@@ -211,46 +213,47 @@ static void create_goto_dialog(void)
 
   rend = gtk_cell_renderer_text_new();
   col = gtk_tree_view_column_new_with_attributes(_("Nation"), rend,
-    "text", GD_COL_NATION, NULL);
+                                                 "text", GD_COL_NATION,
+                                                 NULL);
   gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
   gtk_tree_view_column_set_sort_column_id(col, GD_COL_NATION);
 
   rend = gtk_cell_renderer_text_new();
   col = gtk_tree_view_column_new_with_attributes(_("Airlift"), rend,
-    "text", GD_COL_AIRLIFT, NULL);
+                                                 "text", GD_COL_AIRLIFT,
+                                                 NULL);
   gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
   gtk_tree_view_column_set_sort_column_id(col, GD_COL_AIRLIFT);
 
-  sw = gtk_scrolled_window_new(NULL, NULL);
-  gtk_container_add(GTK_CONTAINER(sw), view);
+  sw = gtk_scrolled_window_new();
+  gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(sw), view);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
-    GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
-  gtk_widget_set_size_request(sw, -1, 200);
+                                 GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
+  gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(sw), 200);
 
-  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
+  gtk_box_append(GTK_BOX(vbox), sw);
 
   all_toggle = gtk_check_button_new_with_mnemonic(_("Show _All Cities"));
-  gtk_box_pack_start(GTK_BOX(vbox), all_toggle, FALSE, FALSE, 0);
+  gtk_box_append(GTK_BOX(vbox), all_toggle);
 
-  g_signal_connect(all_toggle, "toggled", G_CALLBACK(update_goto_dialog), NULL);
+  g_signal_connect(all_toggle, "toggled",
+                   G_CALLBACK(update_goto_dialog), NULL);
 
   g_signal_connect(goto_list_selection, "changed",
-    G_CALLBACK(goto_selection_callback), NULL);
-
-  gtk_widget_show_all(GTK_DIALOG(dshell)->vbox);
-  gtk_widget_show_all(GTK_DIALOG(dshell)->action_area);
+                   G_CALLBACK(goto_selection_callback), NULL);
 
+  gtk_widget_set_visible(dshell, TRUE);
 
   original_tile = get_center_tile_mapcanvas();
 
   update_source_label();
-  update_goto_dialog(GTK_TOGGLE_BUTTON(all_toggle));
+  update_goto_dialog(GTK_CHECK_BUTTON(all_toggle));
   gtk_tree_view_focus(GTK_TREE_VIEW(view));
 }
 
-/****************************************************************
-popup the dialog
-*****************************************************************/
+/**********************************************************************//**
+  Popup the dialog
+**************************************************************************/
 void popup_goto_dialog(void)
 {
   if (!can_client_issue_orders() || get_num_units_in_focus() == 0) {
@@ -264,7 +267,7 @@ void popup_goto_dialog(void)
   gtk_window_present(GTK_WINDOW(dshell));
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Return currently selected city
 **************************************************************************/
 static struct city *get_selected_city(void)
@@ -284,7 +287,7 @@ static struct city *get_selected_city(void)
   return game_city_by_number(city_id);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Appends the list of the city owned by the player in the goto dialog.
 **************************************************************************/
 static bool list_store_append_player_cities(GtkListStore *store,
@@ -316,7 +319,7 @@ static bool list_store_append_player_cities(GtkListStore *store,
   return TRUE;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Refresh the label that shows where the selected unit(s) currently are
   (and the relevant cities' airlift capacities, if relevant).
 **************************************************************************/
@@ -376,7 +379,7 @@ static void update_source_label(void)
 
     astr_init(&strs[i]);
     if (air_text != NULL) {
-      astr_add(&strs[i], 
+      astr_add(&strs[i],
                /* TRANS: goto/airlift dialog. "Paris (airlift: 2/4)".
                 * A set of these appear in an "and"-separated list. */
                _("%s (airlift: %s)"),
@@ -405,7 +408,7 @@ static void update_source_label(void)
   /* Finally, update the label. */
   {
     struct astring label = ASTRING_INIT, list = ASTRING_INIT;
-    astr_set(&label, 
+    astr_set(&label,
              /* TRANS: goto/airlift dialog. Current location of units; %s is an
               * "and"-separated list of cities and associated info */
              _("Currently in: %s"),
@@ -422,21 +425,23 @@ static void update_source_label(void)
   free(last_str); /* might have been NULL */
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Refresh city list (in response to "all cities" checkbox changing).
 **************************************************************************/
-static void update_goto_dialog(GtkToggleButton *button)
+static void update_goto_dialog(GtkCheckButton *button)
 {
   bool nonempty = FALSE;
 
-  gtk_list_store_clear(goto_list_store);
-
   if (!client_has_player()) {
     /* Case global observer. */
     return;
   }
 
-  if (gtk_toggle_button_get_active(button)) {
+  gotodlg_updating = TRUE;
+
+  gtk_list_store_clear(goto_list_store);
+
+  if (gtk_check_button_get_active(button)) {
     players_iterate(pplayer) {
       nonempty |= list_store_append_player_cities(goto_list_store, pplayer);
     } players_iterate_end;
@@ -444,6 +449,8 @@ static void update_goto_dialog(GtkToggleButton *button)
     nonempty |= list_store_append_player_cities(goto_list_store, client_player());
   }
 
+  gotodlg_updating = FALSE;
+
   refresh_airlift_column();
 
   if (!nonempty) {
@@ -453,7 +460,7 @@ static void update_goto_dialog(GtkToggleButton *button)
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Refresh airlift column in city list (without tearing everything down).
 **************************************************************************/
 static void refresh_airlift_column(void)
@@ -478,7 +485,7 @@ static void refresh_airlift_column(void)
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Refresh the state of the "Airlift" button for the currently selected
   unit(s) and city.
 **************************************************************************/
@@ -491,7 +498,7 @@ static void refresh_airlift_button(void)
 
     /* Allow action if any of the selected units can airlift. */
     unit_list_iterate(get_units_in_focus(), punit) {
-      if (unit_can_airlift_to(punit, pdestcity)) {
+      if (unit_can_airlift_to(&(wld.map), punit, pdestcity)) {
         can_airlift = TRUE;
         break;
       }
@@ -506,14 +513,20 @@ static void refresh_airlift_button(void)
   gtk_dialog_set_response_sensitive(GTK_DIALOG(dshell), CMD_AIRLIFT, FALSE);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Update goto dialog. button tells if cities of all players or just
   client's player should be listed.
 **************************************************************************/
 static void goto_selection_callback(GtkTreeSelection *selection,
                                     gpointer data)
 {
-  struct city *pdestcity = get_selected_city();
+  struct city *pdestcity;
+
+  if (gotodlg_updating) {
+    return;
+  }
+
+  pdestcity = get_selected_city();
 
   if (NULL != pdestcity) {
     center_tile_mapcanvas(city_tile(pdestcity));
@@ -521,7 +534,7 @@ static void goto_selection_callback(GtkTreeSelection *selection,
   refresh_airlift_button();
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Called when the set of units in focus has changed; updates airlift info
 **************************************************************************/
 void goto_dialog_focus_units_changed(void)
