@@ -61,7 +61,7 @@ struct tech_class tech_classes[MAX_NUM_TECH_CLASSES];
 
 static struct user_flag user_tech_flags[MAX_NUM_USER_TECH_FLAGS];
 
-/**************************************************************************
+/**********************************************************************//**
   Return the last item of advances/technologies.
 **************************************************************************/
 const struct advance *advance_array_last(void)
@@ -69,10 +69,11 @@ const struct advance *advance_array_last(void)
   if (game.control.num_tech_types > 0) {
     return &advances[game.control.num_tech_types - 1];
   }
-  return NULL;
+
+  return nullptr;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Return the number of advances/technologies.
 **************************************************************************/
 Tech_type_id advance_count_real(void)
@@ -80,7 +81,7 @@ Tech_type_id advance_count_real(void)
   return game.control.num_tech_types;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Return the advance index.
 
   Currently same as advance_number(), paired with advance_count()
@@ -88,20 +89,22 @@ Tech_type_id advance_count_real(void)
 **************************************************************************/
 Tech_type_id advance_index(const struct advance *padvance)
 {
-  fc_assert_ret_val(NULL != padvance, -1);
+  fc_assert_ret_val(padvance != nullptr, -1);
+
   return padvance - advances;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Return the advance index.
 **************************************************************************/
 Tech_type_id advance_number(const struct advance *padvance)
 {
-  fc_assert_ret_val(NULL != padvance, -1);
+  fc_assert_ret_val(padvance != nullptr, -1);
+
   return padvance->item_number;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Return the advance for the given advance index.
 **************************************************************************/
 struct advance *advance_by_number(const Tech_type_id atype)
@@ -109,13 +112,13 @@ struct advance *advance_by_number(const Tech_type_id atype)
   if (atype != A_FUTURE
       && (atype < 0 || atype >= game.control.num_tech_types)) {
     /* This isn't an error; some callers depend on it. */
-    return NULL;
+    return nullptr;
   }
 
   return &advances[atype];
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Accessor for requirements.
 **************************************************************************/
 Tech_type_id advance_required(const Tech_type_id tech,
@@ -130,29 +133,31 @@ Tech_type_id advance_required(const Tech_type_id tech,
   return advance_number(advances[tech].require[require]);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Accessor for requirements.
 **************************************************************************/
 struct advance *advance_requires(const struct advance *padvance,
 				 enum tech_req require)
 {
-  fc_assert_ret_val(require >= 0 && require < AR_SIZE, NULL);
-  fc_assert_ret_val(NULL != padvance, NULL);
+  fc_assert_ret_val(require >= 0 && require < AR_SIZE, nullptr);
+  fc_assert_ret_val(padvance != nullptr, nullptr);
+
   return padvance->require[require];
 }
 
-/**************************************************************************
-  Returns pointer when the advance "exists" in this game, returns NULL
+/**********************************************************************//**
+  Returns pointer when the advance "exists" in this game, returns nullptr
   otherwise.
 
   A tech doesn't exist if it has been flagged as removed by setting its
-  require values to A_NEVER. Note that this function returns NULL if either
-  of req values is A_NEVER, rather than both, to be on the safe side.
+  require values to A_NEVER. Note that this function returns nullptr
+  if either of req values is A_NEVER, rather than both, to be on
+  the safe side.
 **************************************************************************/
 struct advance *valid_advance(struct advance *padvance)
 {
-  if (padvance == NULL) {
-    return NULL;
+  if (padvance == nullptr) {
+    return nullptr;
   }
 
   if (padvance->item_number == A_FUTURE) {
@@ -161,15 +166,15 @@ struct advance *valid_advance(struct advance *padvance)
 
   if (A_NEVER == padvance->require[AR_ONE]
       || A_NEVER == padvance->require[AR_TWO]) {
-    return NULL;
+    return nullptr;
   }
 
   return padvance;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Returns pointer when the advance "exists" in this game,
-  returns NULL otherwise.
+  returns nullptr otherwise.
 
   In addition to valid_advance(), tests for id is out of range.
 **************************************************************************/
@@ -178,40 +183,40 @@ struct advance *valid_advance_by_number(const Tech_type_id id)
   return valid_advance(advance_by_number(id));
 }
 
-/**************************************************************************
- Does a linear search of advances[].name.translated
- Returns NULL when none match.
+/**********************************************************************//**
+  Does a linear search of advances[].name.translated
+  Returns nullptr when none match.
 **************************************************************************/
 struct advance *advance_by_translated_name(const char *name)
 {
-  advance_iterate(A_NONE, padvance) {
+  advance_iterate_all(padvance) {
     if (0 == strcmp(advance_name_translation(padvance), name)) {
       return padvance;
     }
-  } advance_iterate_end;
+  } advance_iterate_all_end;
 
-  return NULL;
+  return nullptr;
 }
 
-/**************************************************************************
- Does a linear search of advances[].name.vernacular
- Returns NULL when none match.
+/**********************************************************************//**
+  Does a linear search of advances[].name.vernacular
+  Returns nullptr when none match.
 **************************************************************************/
 struct advance *advance_by_rule_name(const char *name)
 {
   const char *qname = Qn_(name);
 
-  advance_iterate(A_NONE, padvance) {
+  advance_iterate_all(padvance) {
     if (0 == fc_strcasecmp(advance_rule_name(padvance), qname)) {
       return padvance;
     }
-  } advance_iterate_end;
+  } advance_iterate_all_end;
 
-  return NULL;
+  return nullptr;
 }
 
-/**************************************************************************
- Return TRUE if the tech has this flag otherwise FALSE
+/**********************************************************************//**
+  Return TRUE if the tech has this flag otherwise FALSE
 **************************************************************************/
 bool advance_has_flag(Tech_type_id tech, enum tech_flag_id flag)
 {
@@ -219,15 +224,15 @@ bool advance_has_flag(Tech_type_id tech, enum tech_flag_id flag)
   return BV_ISSET(advance_by_number(tech)->flags, flag);
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Function to precalculate needed data for technologies.
-****************************************************************************/
+**************************************************************************/
 void techs_precalc_data(void)
 {
   fc_assert_msg(tech_cost_style_is_valid(game.info.tech_cost_style),
                 "Invalid tech_cost_style %d", game.info.tech_cost_style);
 
-  advance_iterate(A_FIRST, padvance) {
+  advance_iterate(padvance) {
     int num_reqs = 0;
     bool min_req = TRUE;
 
@@ -264,26 +269,36 @@ void techs_precalc_data(void)
       break;
     }
 
-    if (min_req && padvance->cost < game.info.base_tech_cost) {
-      padvance->cost = game.info.base_tech_cost;
+    if (min_req && padvance->cost < game.info.min_tech_cost) {
+      padvance->cost = game.info.min_tech_cost;
     }
 
     /* Class cost */
-    if (padvance->tclass != NULL) {
+    if (padvance->tclass != nullptr) {
       padvance->cost = padvance->cost * padvance->tclass->cost_pct / 100;
     }
   } advance_iterate_end;
 }
 
-/**************************************************************************
- Is the given tech a future tech.
+/**********************************************************************//**
+  Is the given tech a future tech.
 **************************************************************************/
 bool is_future_tech(Tech_type_id tech)
 {
   return tech == A_FUTURE;
 }
 
-/**************************************************************************
+/**********************************************************************//**
+  Is the given tech an regular advance. "None" counts as regular advance,
+  Future Tech does not.
+**************************************************************************/
+bool is_regular_advance(struct advance *padvance)
+{
+  return valid_advance(padvance) != nullptr
+    && padvance->item_number != A_FUTURE;
+}
+
+/**********************************************************************//**
   Return the (translated) name of the given advance/technology.
   You don't have to free the return pointer.
 **************************************************************************/
@@ -292,16 +307,16 @@ const char *advance_name_translation(const struct advance *padvance)
   return name_translation_get(&padvance->name);
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Return the (untranslated) rule name of the advance/technology.
   You don't have to free the return pointer.
-****************************************************************************/
+**************************************************************************/
 const char *advance_rule_name(const struct advance *padvance)
 {
   return rule_name_get(&padvance->name);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Initialize tech classes
 **************************************************************************/
 void tech_classes_init(void)
@@ -310,23 +325,23 @@ void tech_classes_init(void)
 
   for (i = 0; i < MAX_NUM_TECH_CLASSES; i++) {
     tech_classes[i].idx = i;
-    tech_classes[i].disabled = FALSE;
+    tech_classes[i].ruledit_disabled = FALSE;
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Return the tech_class for the given index.
 **************************************************************************/
 struct tech_class *tech_class_by_number(const int idx)
 {
   if (idx < 0 || idx >= game.control.num_tech_classes) {
-    return NULL;
+    return nullptr;
   }
 
   return &tech_classes[idx];
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Return the (translated) name of the given tech_class
   You must not free the return pointer.
 **************************************************************************/
@@ -335,18 +350,18 @@ const char *tech_class_name_translation(const struct tech_class *ptclass)
   return name_translation_get(&ptclass->name);
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Return the (untranslated) rule name of tech_class
   You must not free the return pointer.
-****************************************************************************/
+**************************************************************************/
 const char *tech_class_rule_name(const struct tech_class *ptclass)
 {
   return rule_name_get(&ptclass->name);
 }
 
-/**************************************************************************
- Does a linear search of tech_classes[].name.vernacular
- Returns NULL when none match.
+/**********************************************************************//**
+  Does a linear search of tech_classes[].name.vernacular
+  Returns nullptr when none match.
 **************************************************************************/
 struct tech_class *tech_class_by_rule_name(const char *name)
 {
@@ -361,10 +376,10 @@ struct tech_class *tech_class_by_rule_name(const char *name)
     }
   }
 
-  return NULL;
+  return nullptr;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Initialize user tech flags.
 **************************************************************************/
 void user_tech_flags_init(void)
@@ -376,9 +391,9 @@ void user_tech_flags_init(void)
   }
 }
 
-/***************************************************************
+/**********************************************************************//**
   Frees the memory associated with all user tech flags
-***************************************************************/
+**************************************************************************/
 void user_tech_flags_free(void)
 {
   int i;
@@ -388,7 +403,7 @@ void user_tech_flags_free(void)
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Sets user defined name for tech flag.
 **************************************************************************/
 void set_user_tech_flag_name(enum tech_flag_id id, const char *name,
@@ -398,18 +413,18 @@ void set_user_tech_flag_name(enum tech_flag_id id, const char *name,
 
   fc_assert_ret(id >= TECH_USER_1 && id <= TECH_USER_LAST);
 
-  if (user_tech_flags[tfid].name != NULL) {
+  if (user_tech_flags[tfid].name != nullptr) {
     FC_FREE(user_tech_flags[tfid].name);
-    user_tech_flags[tfid].name = NULL;
+    user_tech_flags[tfid].name = nullptr;
   }
 
   if (name && name[0] != '\0') {
     user_tech_flags[tfid].name = fc_strdup(name);
   }
 
-  if (user_tech_flags[tfid].helptxt != NULL) {
+  if (user_tech_flags[tfid].helptxt != nullptr) {
     FC_FREE(user_tech_flags[tfid].helptxt);
-    user_tech_flags[tfid].helptxt = NULL;
+    user_tech_flags[tfid].helptxt = nullptr;
   }
 
   if (helptxt && helptxt[0] != '\0') {
@@ -417,19 +432,19 @@ void set_user_tech_flag_name(enum tech_flag_id id, const char *name,
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Tech flag name callback, called from specenum code.
 **************************************************************************/
 const char *tech_flag_id_name_cb(enum tech_flag_id flag)
 {
   if (flag < TECH_USER_1 || flag > TECH_USER_LAST) {
-    return NULL;
+    return nullptr;
   }
 
   return user_tech_flags[flag-TECH_USER_1].name;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Return the (untranslated) helptxt of the user tech flag.
 **************************************************************************/
 const char *tech_flag_helptxt(enum tech_flag_id id)
@@ -439,13 +454,13 @@ const char *tech_flag_helptxt(enum tech_flag_id id)
   return user_tech_flags[id - TECH_USER_1].helptxt;
 }
 
-/**************************************************************************
- Returns true if the costs for the given technology will stay constant
- during the game. False otherwise.
+/**********************************************************************//**
+  Returns true if the costs for the given technology will stay constant
+  during the game. False otherwise.
 
- Checking every tech_cost_style with fixed costs seems a waste of system
- resources, when we can check that it is not the one style without fixed
- costs.
+  Checking every tech_cost_style with fixed costs seems a waste of system
+  resources, when we can check that it is not the one style without fixed
+  costs.
 **************************************************************************/
 bool techs_have_fixed_costs(void)
 {
@@ -453,9 +468,9 @@ bool techs_have_fixed_costs(void)
           && game.info.tech_cost_style != TECH_COST_CIV1CIV2);
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Initialize tech structures.
-****************************************************************************/
+**************************************************************************/
 void techs_init(void)
 {
   struct advance *a_none = &advances[A_NONE];
@@ -465,6 +480,7 @@ void techs_init(void)
   memset(advances, 0, sizeof(advances));
   for (i = 0; i < ARRAY_SIZE(advances); i++) {
     advances[i].item_number = i;
+    advances[i].ruledit_dlg = nullptr;
     advances[i].cost = -1;
     advances[i].inherited_root_req = FALSE;
     advances[i].tclass = 0;
@@ -474,38 +490,38 @@ void techs_init(void)
 
   /* Initialize dummy tech A_NONE */
   /* TRANS: "None" tech */
-  name_set(&a_none->name, NULL, N_("?tech:None"));
+  name_set(&a_none->name, nullptr, N_("?tech:None"));
   a_none->require[AR_ONE] = a_none;
   a_none->require[AR_TWO] = a_none;
   a_none->require[AR_ROOT] = A_NEVER;
 
-  name_set(&a_future->name, NULL, "Future");
+  name_set(&a_future->name, nullptr, "Future");
   a_future->require[AR_ONE] = A_NEVER;
   a_future->require[AR_TWO] = A_NEVER;
   a_future->require[AR_ROOT] = A_NEVER;
 }
 
-/***************************************************************
- De-allocate resources associated with the given tech.
-***************************************************************/
+/**********************************************************************//**
+  De-allocate resources associated with the given tech.
+**************************************************************************/
 static void tech_free(Tech_type_id tech)
 {
   struct advance *p = &advances[tech];
 
-  if (NULL != p->helptext) {
+  if (p->helptext != nullptr) {
     strvec_destroy(p->helptext);
-    p->helptext = NULL;
+    p->helptext = nullptr;
   }
 
   if (p->bonus_message) {
     free(p->bonus_message);
-    p->bonus_message = NULL;
+    p->bonus_message = nullptr;
   }
 }
 
-/***************************************************************
- De-allocate resources of all techs.
-***************************************************************/
+/**********************************************************************//**
+  De-allocate resources of all techs.
+**************************************************************************/
 void techs_free(void)
 {
   int i;
@@ -519,25 +535,25 @@ void techs_free(void)
   }
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Return the size of the advance requirements iterator.
-****************************************************************************/
+**************************************************************************/
 size_t advance_req_iter_sizeof(void)
 {
   return sizeof(struct advance_req_iter);
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Return the current advance.
-****************************************************************************/
+**************************************************************************/
 static void *advance_req_iter_get(const struct iterator *it)
 {
   return (void *) *ADVANCE_REQ_ITER(it)->current;
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Jump to next advance requirement.
-****************************************************************************/
+**************************************************************************/
 static void advance_req_iter_next(struct iterator *it)
 {
   struct advance_req_iter *iter = ADVANCE_REQ_ITER(it);
@@ -547,7 +563,7 @@ static void advance_req_iter_next(struct iterator *it)
 
   for (req = AR_ONE; req < AR_SIZE; req++) {
     preq = valid_advance(advance_requires(padvance, req));
-    if (NULL != preq
+    if (preq != nullptr
         && A_NONE != advance_number(preq)
         && !BV_ISSET(iter->done, advance_number(preq))) {
       BV_SET(iter->done, advance_number(preq));
@@ -565,9 +581,9 @@ static void advance_req_iter_next(struct iterator *it)
   }
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Return whether we finished to iterate or not.
-****************************************************************************/
+**************************************************************************/
 static bool advance_req_iter_valid(const struct iterator *it)
 {
   const struct advance_req_iter *iter = ADVANCE_REQ_ITER(it);
@@ -575,9 +591,9 @@ static bool advance_req_iter_valid(const struct iterator *it)
   return iter->current < iter->end;
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Initialize an advance requirements iterator.
-****************************************************************************/
+**************************************************************************/
 struct iterator *advance_req_iter_init(struct advance_req_iter *it,
                                        const struct advance *goal)
 {
@@ -641,7 +657,7 @@ static void advance_root_req_iter_next(struct iterator *it)
       const struct advance *preq
         = valid_advance(advance_requires(padvance, req));
 
-      if (NULL != preq
+      if (preq != nullptr
           && A_NONE != advance_number(preq)
           && !BV_ISSET(iter->done, advance_number(preq))) {
 
