@@ -1,4 +1,4 @@
-/********************************************************************** 
+/***********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -47,90 +47,38 @@
 #include "techtools.h"
 #include "unittools.h"
 
-/* server/advisors */
-#include "autosettlers.h"
-
 /* server/scripting */
 #include "script_server.h"
 
 #include "diplhand.h"
 
-static struct treaty_list *treaties = NULL;
 
 /* FIXME: Should this be put in a ruleset somewhere? */
 #define TURNS_LEFT 16
 
-/**************************************************************************
-  Calls treaty_evaluate function if such is set for AI player.    
+/**********************************************************************//**
+  Calls treaty_evaluate function if such is set for AI player.
 **************************************************************************/
 static void call_treaty_evaluate(struct player *pplayer, struct player *aplayer,
-                                 struct Treaty *ptreaty)
+                                 struct treaty *ptreaty)
 {
   if (is_ai(pplayer)) {
     CALL_PLR_AI_FUNC(treaty_evaluate, pplayer, pplayer, aplayer, ptreaty);
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Calls treaty_accepted function if such is set for AI player.
 **************************************************************************/
 static void call_treaty_accepted(struct player *pplayer, struct player *aplayer,
-                                 struct Treaty *ptreaty)
+                                 struct treaty *ptreaty)
 {
   if (is_ai(pplayer)) {
     CALL_PLR_AI_FUNC(treaty_accepted, pplayer, pplayer, aplayer, ptreaty);
   }
 }
 
-/**************************************************************************
-  Initialize diplhand module
-**************************************************************************/
-void diplhand_init(void)
-{
-  treaties = treaty_list_new();
-}
-
-/**************************************************************************
-  Free all the resources allocated by diplhand.
-**************************************************************************/
-void diplhand_free(void)
-{
-  free_treaties();
-
-  treaty_list_destroy(treaties);
-  treaties = NULL;
-}
-
-/**************************************************************************
-  Free all the treaties currently in treaty list.
-**************************************************************************/
-void free_treaties(void)
-{
-  /* Free memory allocated for treaties */
-  treaty_list_iterate(treaties, pt) {
-    clear_treaty(pt);
-    free(pt);
-  } treaty_list_iterate_end;
-
-  treaty_list_clear(treaties);
-}
-
-/**************************************************************************
-  Find currently active treaty between two players.
-**************************************************************************/
-struct Treaty *find_treaty(struct player *plr0, struct player *plr1)
-{
-  treaty_list_iterate(treaties, ptreaty) {
-    if ((ptreaty->plr0 == plr0 && ptreaty->plr1 == plr1) ||
-	(ptreaty->plr0 == plr1 && ptreaty->plr1 == plr0)) {
-      return ptreaty;
-    }
-  } treaty_list_iterate_end;
-
-  return NULL;
-}
-
-/**************************************************************************
+/**********************************************************************//**
   Return the closest of the two diplstate types.
 **************************************************************************/
 static enum diplstate_type dst_closest(enum diplstate_type a,
@@ -153,7 +101,22 @@ static enum diplstate_type dst_closest(enum diplstate_type a,
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
+  Return closest diplstate dst has had, with extra sanity checking
+  (i.e. may return state other than the recorder max_state)
+**************************************************************************/
+enum diplstate_type valid_dst_closest(struct player_diplstate *dst)
+{
+  if (dst_closest(dst->type, dst->max_state) == dst->max_state) {
+    /* Sane max_state */
+    return dst->max_state;
+  }
+
+  /* max_state illegally lower than current. Return current as max_state */
+  return dst->type;
+}
+
+/**********************************************************************//**
   Set diplstate between two players.
 **************************************************************************/
 void set_diplstate_type(struct player_diplstate *state1,
@@ -176,7 +139,7 @@ void set_diplstate_type(struct player_diplstate *state1,
   state2->max_state = max;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   pplayer clicked the accept button. If they accepted the treaty we check
   the clauses. If both players have now accepted the treaty we execute the
   agreed clauses.
@@ -184,12 +147,15 @@ void set_diplstate_type(struct player_diplstate *state1,
 void handle_diplomacy_accept_treaty_req(struct player *pplayer,
                                         int counterpart)
 {
-  struct Treaty *ptreaty;
+  struct treaty *ptreaty;
   bool *player_accept, *other_accept;
   enum dipl_reason diplcheck;
   bool worker_refresh_required = FALSE;
   struct player *pother = player_by_number(counterpart);
 
+  const struct req_context player_ctxt = { .player = pplayer };
+  const struct req_context other_ctxt  = { .player = pother };
+
   if (NULL == pother || pplayer == pother) {
     return;
   }
@@ -210,13 +176,26 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
 
   if (!*player_accept) {	/* Tries to accept. */
 
-    /* Check that player who accepts can keep what (s)he promises. */
+    /* Check that player who accepts can keep what they promise. */
 
     clause_list_iterate(ptreaty->clauses, pclause) {
       struct city *pcity = NULL;
 
+      if (pclause->from == pplayer) {
+        struct clause_info *info = clause_info_get(pclause->type);
+
+        if (!are_reqs_active(&player_ctxt, &other_ctxt,
+                             &(info->giver_reqs), RPT_POSSIBLE)
+            || !are_reqs_active(&other_ctxt, &player_ctxt,
+                                &(info->receiver_reqs), RPT_POSSIBLE)) {
+          log_error("Requirements of a clause between %s and %s not fulfilled",
+                    player_name(pplayer), player_name(pother));
+          return;
+        }
+      }
+
       if (pclause->from == pplayer || is_pact_clause(pclause->type)) {
-	switch(pclause->type) {
+	switch (pclause->type) {
 	case CLAUSE_EMBASSY:
           if (player_has_real_embassy(pother, pplayer)) {
             log_error("%s tried to give embassy to %s, who already "
@@ -312,9 +291,13 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
 	    return;
 	  }
 	  break;
-	default:
-	  ; /* nothing */
-	}
+        case CLAUSE_MAP:
+        case CLAUSE_SEAMAP:
+        case CLAUSE_VISION:
+        case CLAUSE_SHARED_TILES:
+        case CLAUSE_COUNT:
+          break;
+        }
       }
     } clause_list_iterate_end;
   }
@@ -350,10 +333,30 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
                   nclauses);
 
     /* Check that one who accepted treaty earlier still have everything
-       (s)he promised to give. */
+       they promised to give. */
 
     clause_list_iterate(ptreaty->clauses, pclause) {
       struct city *pcity;
+
+      if (pclause->from == pother) {
+        struct clause_info *info = clause_info_get(pclause->type);
+
+        if (!are_reqs_active(&other_ctxt, &player_ctxt,
+                             &(info->giver_reqs), RPT_POSSIBLE)
+            || !are_reqs_active(&player_ctxt, &other_ctxt,
+                                &(info->receiver_reqs), RPT_POSSIBLE)) {
+          notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
+                        _("Clause requirements are no longer fulfilled. "
+                          "Treaty with %s canceled!"),
+                        nation_plural_for_player(pother));
+          notify_player(pother, NULL, E_DIPLOMACY, ftc_server,
+                        _("Clause requirements are no longer fulfilled. "
+                          "Treaty with %s canceled!"),
+                        nation_plural_for_player(pplayer));
+          return;
+        }
+      }
+
       if (pclause->from == pother) {
 	switch (pclause->type) {
 	case CLAUSE_CITY:
@@ -563,9 +566,8 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
           pgiver_seen_units = get_units_seen_via_ally(pgiver, pdest);
           pdest_seen_units = get_units_seen_via_ally(pdest, pgiver);
         }
-        ds_giverdest->type = DS_CEASEFIRE;
+        set_diplstate_type(ds_giverdest, ds_destgiver, DS_CEASEFIRE);
         ds_giverdest->turns_left = TURNS_LEFT;
-        ds_destgiver->type = DS_CEASEFIRE;
         ds_destgiver->turns_left = TURNS_LEFT;
         notify_player(pgiver, NULL, E_TREATY_CEASEFIRE, ftc_server,
                       _("You agree on a cease-fire with %s."),
@@ -655,6 +657,19 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
          * within radius of our own city. */
         worker_refresh_required = TRUE;
 	break;
+      case CLAUSE_SHARED_TILES:
+        BV_SET(pgiver->gives_shared_tiles, player_index(pdest));
+        notify_player(pgiver, NULL, E_TREATY_SHARED_TILES, ftc_server,
+                      _("You share your tiles with %s."),
+                      player_name(pdest));
+        notify_player(pdest, NULL, E_TREATY_SHARED_TILES, ftc_server,
+                      _("%s shares their tiles with you."),
+                      player_name(pgiver));
+        worker_refresh_required = TRUE;
+        break;
+      case CLAUSE_COUNT:
+        fc_assert(pclause->type != CLAUSE_COUNT);
+        break;
       }
 
     } clause_list_iterate_end;
@@ -671,17 +686,15 @@ void handle_diplomacy_accept_treaty_req(struct player *pplayer,
     }
 
   cleanup:
-    treaty_list_remove(treaties, ptreaty);
-    clear_treaty(ptreaty);
-    free(ptreaty);
+    treaty_remove(ptreaty);
     send_player_all_c(pplayer, NULL);
     send_player_all_c(pother, NULL);
   }
 }
 
-/****************************************************************************
+/**********************************************************************//**
   Create an embassy. pplayer gets an embassy with aplayer.
-****************************************************************************/
+**************************************************************************/
 void establish_embassy(struct player *pplayer, struct player *aplayer)
 {
   /* Establish the embassy. */
@@ -701,14 +714,14 @@ void establish_embassy(struct player *pplayer, struct player *aplayer)
   send_research_info(research_get(aplayer), pplayer->connections);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Handle request to remove clause from treaty.
 **************************************************************************/
 void handle_diplomacy_remove_clause_req(struct player *pplayer,
-					int counterpart, int giver,
-					enum clause_type type, int value)
+                                        int counterpart, int giver,
+                                        enum clause_type type, int value)
 {
-  struct Treaty *ptreaty;
+  struct treaty *ptreaty;
   struct player *pgiver = player_by_number(giver);
   struct player *pother = player_by_number(counterpart);
 
@@ -719,7 +732,7 @@ void handle_diplomacy_remove_clause_req(struct player *pplayer,
   if (pgiver != pplayer && pgiver != pother) {
     return;
   }
-  
+
   ptreaty = find_treaty(pplayer, pother);
 
   if (ptreaty && remove_clause(ptreaty, pgiver, type, value)) {
@@ -734,14 +747,14 @@ void handle_diplomacy_remove_clause_req(struct player *pplayer,
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Handle request to add clause to treaty between two players.
 **************************************************************************/
 void handle_diplomacy_create_clause_req(struct player *pplayer,
-					int counterpart, int giver,
-					enum clause_type type, int value)
+                                        int counterpart, int giver,
+                                        enum clause_type type, int value)
 {
-  struct Treaty *ptreaty;
+  struct treaty *ptreaty;
   struct player *pgiver = player_by_number(giver);
   struct player *pother = player_by_number(counterpart);
 
@@ -755,11 +768,12 @@ void handle_diplomacy_create_clause_req(struct player *pplayer,
 
   ptreaty = find_treaty(pplayer, pother);
 
-  if (ptreaty && add_clause(ptreaty, pgiver, type, value)) {
-    /* 
-     * If we are trading cities, then it is possible that the
-     * dest is unaware of it's existence.  We have 2 choices,
-     * forbid it, or lighten that area.  If we assume that
+  if (ptreaty != NULL
+      && add_clause(ptreaty, pgiver, type, value, NULL)) {
+    /*
+     * If we are trading cities, then it is possible that
+     * the dest is unaware of it's existence. We have 2 choices,
+     * forbid it, or lighten that area. If we assume that
      * the giver knows what they are doing, then 2. is the
      * most powerful option - I'll choose that for now.
      *                           - Kris Bubendorfer
@@ -767,8 +781,9 @@ void handle_diplomacy_create_clause_req(struct player *pplayer,
     if (type == CLAUSE_CITY) {
       struct city *pcity = game_city_by_number(value);
 
-      if (pcity && !map_is_known_and_seen(pcity->tile, pother, V_MAIN))
+      if (pcity && !map_is_known_and_seen(pcity->tile, pother, V_MAIN)) {
 	give_citymap_from_player_to_player(pcity, pplayer, pother);
+      }
     }
 
     dlsend_packet_diplomacy_create_clause(pplayer->connections,
@@ -782,36 +797,34 @@ void handle_diplomacy_create_clause_req(struct player *pplayer,
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Cancel meeting. No sanity checking of input parameters, so don't call
   this with input directly from untrusted source.
 **************************************************************************/
 static void really_diplomacy_cancel_meeting(struct player *pplayer,
-					    struct player *pother)
+                                            struct player *pother)
 {
-  struct Treaty *ptreaty = find_treaty(pplayer, pother);
+  struct treaty *ptreaty = find_treaty(pplayer, pother);
 
   if (ptreaty) {
     dlsend_packet_diplomacy_cancel_meeting(pother->connections,
 					   player_number(pplayer),
 					   player_number(pplayer));
     notify_player(pother, NULL, E_DIPLOMACY, ftc_server,
-                  _("%s canceled the meeting!"), 
+                  _("%s canceled the meeting!"),
                   player_name(pplayer));
     /* Need to send to pplayer too, for multi-connects: */
     dlsend_packet_diplomacy_cancel_meeting(pplayer->connections,
 					   player_number(pother),
 					   player_number(pplayer));
     notify_player(pplayer, NULL, E_DIPLOMACY, ftc_server,
-                  _("Meeting with %s canceled."), 
+                  _("Meeting with %s canceled."),
                   player_name(pother));
-    treaty_list_remove(treaties, ptreaty);
-    clear_treaty(ptreaty);
-    free(ptreaty);
+    treaty_remove(ptreaty);
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Handle meeting cancelling request.
 **************************************************************************/
 void handle_diplomacy_cancel_meeting_req(struct player *pplayer,
@@ -826,11 +839,11 @@ void handle_diplomacy_cancel_meeting_req(struct player *pplayer,
   really_diplomacy_cancel_meeting(pplayer, pother);
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Handle meeting opening request.
 **************************************************************************/
 void handle_diplomacy_init_meeting_req(struct player *pplayer,
-				       int counterpart)
+                                       int counterpart)
 {
   struct player *pother = player_by_number(counterpart);
 
@@ -850,11 +863,11 @@ void handle_diplomacy_init_meeting_req(struct player *pplayer,
   }
 
   if (could_meet_with_player(pplayer, pother)) {
-    struct Treaty *ptreaty;
+    struct treaty *ptreaty;
 
     ptreaty = fc_malloc(sizeof(*ptreaty));
     init_treaty(ptreaty, pplayer, pother);
-    treaty_list_prepend(treaties, ptreaty);
+    treaty_add(ptreaty);
 
     dlsend_packet_diplomacy_init_meeting(pplayer->connections,
 					 player_number(pother),
@@ -865,7 +878,7 @@ void handle_diplomacy_init_meeting_req(struct player *pplayer,
   }
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Send information on any on-going diplomatic meetings for connection's
   player.  For re-connections.
 **************************************************************************/
@@ -877,14 +890,14 @@ void send_diplomatic_meetings(struct connection *dest)
     return;
   }
   players_iterate(other) {
-    struct Treaty *ptreaty = find_treaty(pplayer, other);
+    struct treaty *ptreaty = find_treaty(pplayer, other);
 
     if (ptreaty) {
       fc_assert_action(pplayer != other, continue);
       dsend_packet_diplomacy_init_meeting(dest, player_number(other),
                                           player_number(pplayer));
       clause_list_iterate(ptreaty->clauses, pclause) {
-        dsend_packet_diplomacy_create_clause(dest, 
+        dsend_packet_diplomacy_create_clause(dest,
                                              player_number(other),
                                              player_number(pclause->from),
                                              pclause->type,
@@ -904,7 +917,7 @@ void send_diplomatic_meetings(struct connection *dest)
   } players_iterate_end;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Cancels all meetings of player.
 **************************************************************************/
 void cancel_all_meetings(struct player *pplayer)
@@ -916,15 +929,16 @@ void cancel_all_meetings(struct player *pplayer)
   } players_iterate_end;
 }
 
-/**************************************************************************
+/**********************************************************************//**
   Reject all treaties currently being negotiated
 **************************************************************************/
 void reject_all_treaties(struct player *pplayer)
 {
-  struct Treaty* treaty;
+  struct treaty *treaty;
+
   players_iterate(pplayer2) {
     treaty = find_treaty(pplayer, pplayer2);
-    if (!treaty) {
+    if (treaty == nullptr) {
       continue;
     }
     treaty->accept0 = FALSE;
@@ -939,11 +953,3 @@ void reject_all_treaties(struct player *pplayer)
 					  FALSE);
   } players_iterate_end;
 }
-
-/**************************************************************************
-  Get treaty list
-**************************************************************************/
-struct treaty_list *get_all_treaties(void)
-{
-  return treaties;
-}
